@@ -7,13 +7,6 @@ use DADA::Config qw(!:DEFAULT);
 use DADA::App::Guts; 
 use DADA::MailingList::Settings;
 
-# Sorry - global!
-my $List_Names = {};
-foreach my $l( available_lists() ){
-	my $ls = DADA::MailingList::Settings->new({-list => $l}); 
-		$List_Names->{$l} = $ls->param('list_name');				
-}
-
 use Carp qw(croak carp);
 use Fcntl qw(	O_WRONLY	O_TRUNC		O_CREAT		);
 
@@ -46,6 +39,8 @@ sub AUTOLOAD {
     my $self = shift; 
     my $type = ref($self) 
     	or croak "$self is not an object"; 
+
+	return if(substr($AUTOLOAD, -7) eq 'DESTROY');
     	
     my $name = $AUTOLOAD;
        $name =~ s/.*://; #strip fully qualifies portion 
@@ -66,7 +61,12 @@ sub AUTOLOAD {
 sub _init { 
 
 	my $self = shift; 
-		
+	
+    foreach my $l( available_lists() ){
+    	my $ls = DADA::MailingList::Settings->new({-list => $l}); 
+    		$self->{List_Names}->{$l} = $ls->param('list_name');				
+    }
+    
 }
 
 
@@ -231,6 +231,7 @@ sub log_line_report {
         moderators         => 'Moderators',
         white_list         => 'White Listed',
         sub_request_list   => 'Subscription Requests',
+        unsub_request_list => 'Unsubscription Requests',
         bounced_list       => 'Bouncing Addresses',
 		invitelist         => 'List Invitations', 
     );
@@ -247,7 +248,6 @@ sub log_line_report {
 		
 	if(exists($args->{-list})){ 	
 		if($list ne $args->{-list}) { 
-			#warn 'here.' . $args->{-line}; 
 			return {};
 		} 
 	}
@@ -301,6 +301,7 @@ sub log_line_report {
 
     # Subscribed to announce.sub_confirm_list	nillajess@yahoo.com
 
+	
     if ( $action =~ m/Subscribed to/ ) {
         $action =~ m/^Subscribed to $list\.(.*?)$/;
         $base_action = 'subscribed';
@@ -328,18 +329,17 @@ sub log_line_report {
 
     $date =~ s/(\[|\])//g;
 
-    return {
-        date           => $date,
-        list           => $list,
-		list_name      => $List_Names->{$list}, 
-        ip             => $ip,
-        email          => $email,
-        type           => $sublist,
-        type_title     => $list_types{$sublist},
-        action         => $base_action,
-		updated_email  => $new_email, # used for subscription updates
-    };
-
+	    return {
+	        date           => $date,
+	        list           => $list,
+			list_name      => $self->{List_Names}->{$list}, 
+	        ip             => $ip,
+	        email          => $email,
+	        type           => $sublist,
+	        type_title     => $list_types{$sublist},
+	        action         => $base_action,
+			updated_email  => $new_email, # used for subscription updates
+	    };
 }
 
 sub sub_unsub_trends { 
@@ -480,11 +480,11 @@ sub sub_unsub_trends_json {
 		my $datatable = Data::Google::Visualization::DataTable->new();
 
 		$datatable->add_columns(
-			   { id => 'date',                    label => 'Date',                    type => 'string'}, 
-			   { id => 'cumulative_subscribed',   label => 'Cumulative Subscriptions',   type => 'number',},
+			   { id => 'date',                    label => 'Date',                      type => 'string'}, 
+			   { id => 'cumulative_subscribed',   label => 'Cumulative Subscriptions',  type => 'number',},
 			   { id => 'cumulative_unsubscribed', label => 'Cumulative Unubscriptions', type => 'number',},
-			   { id => 'subscribed',              label => 'Subscriptions',   type => 'number',},
-			   { id => 'unsubscribed',            label => 'Unubscriptions', type => 'number',},
+			   { id => 'subscribed',              label => 'Subscriptions',             type => 'number',},
+			   { id => 'unsubscribed',            label => 'Unubscriptions',            type => 'number',},
 		);
 
 		for(@$trends){ 
@@ -598,7 +598,7 @@ sub html_highlight_line {
     my @terms = split(' ', $args->{-query}); 
     
     for my $term(@terms){  
-        $args->{-line} =~ s{$term}{<em class="highlighted">$term</em>}mg;
+        $args->{-line} =~ s{$term}{<em class="dm_highlighted">$term</em>}mg;
     }
     
    return $args->{-line}; 
@@ -740,9 +740,9 @@ Justin Simoni
 
 See: http://dadamailproject.com/contact
 
-=head1 LICENCE AND COPYRIGHT
+=head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2006 - 2011 Justin Justin Simoni All rights reserved. 
+Copyright (c) 2006 - 2015 Justin Justin Simoni All rights reserved. 
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
