@@ -6,8 +6,9 @@ use DADA::App::Guts;
 use Carp qw(carp croak); 
 use Try::Tiny; 
 
+my $t = $DADA::Config::DEBUG_TRACE->{DADA_MailingList_Archives};
+
 
-# A weird fix.
 BEGIN {
    if($] > 5.008){
       require Errno;
@@ -70,36 +71,38 @@ what the previous and next entry keys are.
 
 =cut
 
-sub get_neighbors { 
-	my $self    = shift; 
-	my $key     = shift;
-	   $key     = $self->_massaged_key($key); 
-	my $entries = $self->get_archive_entries(); 
-	
-	my $i = 0; 
-	my %lookup_hash; 
-	
-	for(@$entries){ 
-		$lookup_hash{$_} = $i;
-		$i++;  
-	}
+sub get_neighbors {
+    my $self = shift;
+    my $key  = shift;
+    $key = $self->_massaged_key($key);
+    my $entries = $self->get_archive_entries();
 
-	# oh, I get it - it's in reverse chrono...
-	my $index = $lookup_hash{$key}; 	
-	my $prev  = $entries->[$index-1]
-		if $entries->[$index-1] != $entries->[-1];
-	my $next  = $entries->[$index+1]
-		if $entries->[$index+1];
-		 # That wraparound stuff is stupid.
-	#my $next  = $entries -> [$index+1] || $entries->[0]; 
-	
-	if($self->{ls}->param('sort_archives_in_reverse')){ 
-		return ($next, $prev); 
-	}else{ 
-	
-		return ($prev, $next); 
-	
-	}
+    my $i = 0;
+    my %lookup_hash;
+
+    for (@$entries) {
+        $lookup_hash{$_} = $i;
+        $i++;
+    }
+
+    # oh, I get it - it's in reverse chrono...
+    my $index = $lookup_hash{$key};
+    my $prev  = $entries->[ $index - 1 ]
+      if $entries->[ $index - 1 ] != $entries->[-1];
+    my $next = $entries->[ $index + 1 ]
+      if $entries->[ $index + 1 ];
+
+    # That wraparound stuff is stupid.
+    #my $next  = $entries -> [$index+1] || $entries->[0];
+
+    if ( $self->{ls}->param('sort_archives_in_reverse') ) {
+        return ( $next, $prev );
+    }
+    else {
+
+        return ( $prev, $next );
+
+    }
 }
 
 
@@ -292,17 +295,12 @@ sub get_header {
 				); 
 
 	my ($subject, $message, $format, $raw_msg) = $self->get_archive_info($args{-key}, 1); 
-	
-	my $entity = $self->_entity_from_raw_msg($raw_msg); 
-			
+	my $entity = $self->_entity_from_raw_msg($raw_msg); 			
 	my $header = $entity->head->get($args{-header}, 0); 
 
-
 	# DEV: Decode Header!
-	if($args{-header} =~ m/Reply\-To|To|From|Subject|Cc/){ 
-		
+	if($args{-header} =~ m/Reply\-To|To|From|Subject|Cc|Sender|X\-Origina\-From/){ 
 		$header = $self->_decode_header($header); 
-#		die $header; 
 	}
 
 	#Special Case. 
@@ -310,13 +308,46 @@ sub get_header {
 		$header = $self->strip_subjects_appended_list_name($header)
 			if $self->{ls}->param('no_prefix_list_name_to_subject_in_archives') == 1; 
 	}
-
 	
 	
+	$entity->purge;
+	undef($entity);
+	
 	return $header; 
 	
 }
 
+sub sender_address {
+
+    my $self = shift; 
+    my ($args) = @_; 
+    my $id = $args->{-id};
+    my $header = $args->{-header}; 
+    
+    require Email::Address;
+    
+    my $e = $self->get_header(
+        -key    => $id,
+        -header => 'X-Original-From', 
+    );
+    if(!defined($e) || length($e) <= 1){ 
+        $e = $self->get_header(
+            -key    => $id,
+            -header => 'From', 
+        );
+    }
+    my $a; 
+    
+    if(defined($e) && length($e) > 0){ 
+        $a = ( Email_Address_parse($e) )[0]->address;
+    }
+    else { 
+        warn 'no sender address found.'; 
+    }
+    return $a; 
+    
+}
+
 
 
 
@@ -482,74 +513,84 @@ at the bottom of each archive index
 # This looks like it creates navigation for lists of archives, rather than for individual archives...
 # One day, this should be replaced with Data::Pageset
 
-sub create_index_nav { 
+sub create_index_nav {
 
-	my $self          = shift; 
-	my $stopped_at    = shift; 
-	my $admin         = shift || 0; 
-	
-	my $iterate       = $self->{ls}->param('archive_index_count') || 10;
-	my $entries       = $self->get_archive_entries(); 
-	my $forward       = $stopped_at;
-	my $back; 
-	
-	# let see if we're at some weird halfway between point
-	
-	
-	my $mod_check     = $stopped_at % $iterate; 
-	my $fixer; 
-	
-	my $full_stop     = $stopped_at; 
+    my $self       = shift;
+    my $stopped_at = shift;
+    my $admin      = shift || 0;
 
-	if($mod_check > 0){ 
-		# substract it from the iterate 
-		$fixer      = $iterate - $mod_check; 		
-		$full_stop +=  $fixer; 		
-	}
-	
-	$back = ($full_stop - ($iterate*2)); 	
-	
-	my $prev_link = ''; 
-	my $next_link = '';
-	
-	my $af;
-	
-	if($admin == 1){ 
-		$af = 'view_archive'; 
-	}else{ 
-		$af = 'archive'; 
-	}
-	
-	my $prev_link_start; 
-	my $next_link_start; 
-	
-	if($self->{ls}->param('sort_archives_in_reverse')){ 
-		if($back >= 0){		
-			$next_link_start = $back; 
-		}
-		if(($forward-1)  < $#{$entries}){
-			$prev_link_start = $forward;
-		}
-	}
-	else { 
-		if($back >= 0){		
-			$prev_link_start = $back; 
-		}
-				
-		if(($forward-1)  < $#{$entries}){
-			$next_link_start = $forward;
-		}		
-	}
-	
-	require DADA::Template::Widgets;
+    my $iterate = $self->{ls}->param('archive_index_count') || 10;
+    my $entries = $self->get_archive_entries();
+    my $forward = $stopped_at;
+    my $back;
+
+    # let see if we're at some weird halfway between point
+
+    my $mod_check = $stopped_at % $iterate;
+    my $fixer;
+
+    my $full_stop = $stopped_at;
+
+    my $url = $DADA::Config::PROGRAM_URL;
+    if($admin == 1){ 
+        $url = $DADA::Config::S_PROGRAM_URL;
+    }
+    
+    if ( $mod_check > 0 ) {
+
+        # substract it from the iterate
+        $fixer = $iterate - $mod_check;
+        $full_stop += $fixer;
+    }
+
+    $back = ( $full_stop - ( $iterate * 2 ) );
+
+    my $prev_link = '';
+    my $next_link = '';
+
+    my $af;
+
+    if ( $admin == 1 ) {
+        $af  = 'view_archive';
+        $url = $DADA::Config::S_PROGRAM_URL;
+    }
+    else {
+        $af = 'archive';
+    }
+
+    my $prev_link_start;
+    my $next_link_start;
+
+    if ( $self->{ls}->param('sort_archives_in_reverse') ) {
+        if ( $back >= 0 ) {
+            $next_link_start = $back;
+        }
+        if ( ( $forward - 1 ) < $#{$entries} ) {
+            $prev_link_start = $forward;
+        }
+    }
+    else {
+        if ( $back >= 0 ) {
+            $prev_link_start = $back;
+        }
+
+        if ( ( $forward - 1 ) < $#{$entries} ) {
+            $next_link_start = $forward;
+        }
+    }
+    
+
+    require DADA::Template::Widgets;
     return DADA::Template::Widgets::screen(
         {
             -screen => 'archive_index_nav_table_widget.tmpl',
             -list   => $self->{name},
+            -expr   => 1, 
             -vars   => {
-				flavor_label       => $af, 
-				prev_link_start    => $prev_link_start, 
-				next_link_start    => $next_link_start, 
+                url             => $url,
+                flavor_label    => $af,
+                prev_link_start => $prev_link_start,
+                next_link_start => $next_link_start,
             },
             -list_settings_vars       => $self->{ls}->params,
             -list_settings_vars_param => { -dot_it => 1 },
@@ -636,38 +677,6 @@ sub make_nav_table {
 }
 
 
-
-
-
-=pod
-
-=head2 make_search_form
-
-	print $archive -> make_search_form(); 
-
-this prints out the correct HTML form to make for your archives. 
-
-=cut
-
-
-sub make_search_form { 
-	
-	my $self = shift; 
-	
-	# ?!?!
-	my $list = shift; 
-				
-	require DADA::Template::Widgets; 
-	return  DADA::Template::Widgets::screen(
-	            {
-	                -screen => 'archive_search_form.tmpl', 
-					-list   => $list
-			    }
-			);
-}
-
-
-
 sub _neuter_confirmation_token_links { 
 	my $self = shift; 
 	my $body = shift; 
@@ -683,27 +692,6 @@ sub _neuter_confirmation_token_links {
 	
 }
 
-sub _scrub_js { 
-
-	my $self = shift; 
-	my $body = shift; 
-
-	try {  
-		require HTML::Scrubber;
-		my $scrubber = HTML::Scrubber->new(
-	    	%{$DADA::Config::HTML_SCRUBBER_OPTIONS}
-	    );
-		$body = $scrubber->scrub($body); 
-	} catch { 
-		carp "Cannot use HTML::Scrubber: $_"; 
-	};
-	
-	return $body;	
-}
-
-
-
-
 sub _email_protect { 
 	
 	my $self   = shift; 
@@ -742,6 +730,10 @@ sub _email_protect {
 			
 			
             $body =~ s/$look_e/$protected_e/g;
+		}
+		elsif($self->{ls}->param('archive_protect_email') eq 'break'){ 				
+            my $protected_e = break_encode($_); 
+              $body =~ s/$look_e/$protected_e/g;
 		}
 		elsif($self->{ls}->param('archive_protect_email') eq 'spam_me_not'){ 				
             my $protected_e = spam_me_not_encode($_); 
@@ -784,6 +776,13 @@ sub _email_protect {
             $body =~ s/$le/$pe/g;
             
 		}
+		elsif($self->{ls}->param('archive_protect_email') eq 'break'){ 				
+            my $pe = break_encode($fa); 
+            my $le = quotemeta($fa); 
+			$body =~ s/$le/$pe/g;
+		}
+		
+		
 		elsif($self->{ls}->param('archive_protect_email') eq 'spam_me_not'){ 		
             my $pe = spam_me_not_encode($fa);
             my $le = quotemeta($fa); 
@@ -791,6 +790,8 @@ sub _email_protect {
         }
 	}
  
+	$entity->purge;
+ 	undef($entity); 
 	
    return $body; 
 
@@ -836,6 +837,10 @@ sub _zap_sig_plaintext {
 	my $message     = shift; 
 	my @msg_lines   = split(/\n/, $message);
 
+	# This message is blank,
+	return $message 
+		if scalar @msg_lines <= 0; 
+		
 	my $new_message = undef; 
 	
 	my $start       = 0; 
@@ -910,8 +915,25 @@ sub _zap_sig_html {
 	# This stops us from chopping off anything that's also on the same line 
 	# as the opening/sig comments. Tricky. 
 	
-	$message =~ s/($opening_open|$opening_close|$sig_open|$sig_close)/\n$1\n/g;
+	my $tag_check = 1; 
+	if($message =~ m/$opening_open/ && $message =~ m/$opening_close/){ 
+		# ...
+	}
+	else { 		
+		$tag_check = 0; 
+	}
 	
+	if($message =~ m/$sig_open/ && $message =~ m/$sig_close/){ 
+		# ... 
+	}
+	else { 
+		$tag_check = 0; 
+	}
+	
+	return $message
+		if $tag_check == 0; 
+	
+	$message =~ s/($opening_open|$opening_close|$sig_open|$sig_close)/\n$1\n/g;
 	
 	
 	my @msg_lines   = split(/\n/, $message);
@@ -919,6 +941,8 @@ sub _zap_sig_html {
 	
 	my $switch = 1; 
 	
+	# DEV TODO: really, this should take two passe
+	
     for my $line(@msg_lines){ 
 		if($line =~ m/$opening_open/ || $line =~ m/$sig_open/){  
 			$switch = 0;
@@ -938,6 +962,7 @@ sub _zap_sig_html {
     
 	}
 	
+	#warn '$new_message' . $new_message; 
 	if(! $new_message){ 
 		return $message;
 	}else{ 
@@ -946,6 +971,53 @@ sub _zap_sig_html {
 }
 
 
+sub string_between_markers { 
+
+	my $self        = shift; 
+	my $str         = shift; 
+	my $begin       = shift || '<!-- start message content -->'; 
+	my $end         = shift || '<!-- end message content -->';
+	
+
+	
+	my $qm_begin = quotemeta($begin);
+	my $qm_end = quotemeta($end);
+	
+	$str =~ s/($qm_begin|$qm_end)/\n$1\n/g;
+	
+	my @str_lines   = split(/\n|\r/, $str);
+	my $n_str = undef; 
+	
+	my $switch = 0; 
+	
+    for my $line(@str_lines){ 
+		if($line =~ m/$qm_begin/){  
+			$switch = 1;
+			next; 
+		}
+		
+		if($line =~ m/$qm_end/){  
+    		$switch = 0;
+    		next;
+   
+    	}
+    	next 
+    		if $switch == 0; 
+    	
+    	$n_str .= $line . "\n"; 
+    
+	}
+	
+
+	
+	if(! $n_str){ 
+		return $str;
+	}else{ 
+		return $n_str; 
+	}
+}
+
+
 
 
 sub _highlight_quoted_text { 
@@ -963,14 +1035,14 @@ sub _highlight_quoted_text {
 		chomp($line); 
 		# line begins with a, ">"
 		if($line =~ m/^\&gt;/){
-			$line =~ s/^\&gt;//; 
+			#$line =~ s/^\&gt;//; 
 			
 			if($line =~ m/\<BR\>$/i){ 
 				$br_at_end = 1; 
 				$line =~ s/\<BR\>$//i;
 			}
 			
-			$line = '<span class="quoted_reply">&gt;' . $line . '</span>';
+			$line = '<blockquote>' . $line . '</blockquote>';
 			$line .= '<br />'
 				if $br_at_end == 1; 
 		}
@@ -994,21 +1066,23 @@ sub _entity_from_raw_msg {
 	}
 
 	my $entity; 
-	eval { $entity = $self->{parser}->parse_data(
+	try { $entity = $self->{parser}->parse_data(
 			safely_encode( 
 				$raw_msg 
 			)
-		) };
-	if($@){ 
-		croak "Problems creating entity: $@"; 
-	}
+		) 
+	} catch { 
+
+		croak "Problems creating entity: $_"; 
+	};
 	
 	if(!$entity){
 		carp "Couldn't create an entity from this raw_msg:\n$raw_msg";
 		return undef;
 	}
-	
-	return $entity; 
+	else {
+		return $entity; 
+	}
 }
 
 
@@ -1038,6 +1112,9 @@ sub attachment_list {
 	my $entity      = $self->_entity_from_raw_msg($raw_msg);
 	my $attachments = $self->find_attachment_list($entity, $key); 
 
+	$entity->purge;
+ 	undef($entity); 
+
 	return $attachments; 
 }
 
@@ -1081,11 +1158,25 @@ sub find_attachment_list {
 				   $entity->head->mime_attr("content-disposition.filename");
 		
 		if($name){ 			
-			push(@$attachment_list, {name => $name, list => $self->{name}, id => $id, PROGRAM_URL => $DADA::Config::PROGRAM_URL });
+			push(
+				@$attachment_list, 
+				{
+					original_name  => $name, 
+					name           => $self->_decode_header($name), 
+					uriescape_name => uriescape($self->_decode_header($name)),
+					list           => $self->{name}, 
+					id             => $id, 
+					PROGRAM_URL     => $DADA::Config::PROGRAM_URL 
+				}
+			);
 		}else{ 
 			#warn "no name?!"; 
 		}	
 	}
+	
+	$entity->purge;
+ 	undef($entity); 
+	
 	return $attachment_list; 
 }
 
@@ -1093,7 +1184,7 @@ sub find_attachment_list {
 
 
 sub view_file_attachment { 
-	
+    	
 	my $self = shift;
 
 	my %args = (
@@ -1114,19 +1205,18 @@ sub view_file_attachment {
 	
 	my $r; 
 	
-	require CGI; 
-	my $q = CGI->new; 
-	   $q->charset($DADA::Config::HTML_CHARSET);
-	   $q = decode_cgi_obj($q);
 	
 	my ($subject, $message, $format, $raw_msg) = $self->get_archive_info($id, 1); 
 	
 	if(! $raw_msg){ 
-		$r.= return $q->header('text/plain') . "can not find the attachment in question."; 
+		return ({-type => 'text/plain'}, "can not find the attachment in question."); 
 	}
 	
 	my $entity   = $self->_entity_from_raw_msg($raw_msg);
 	
+	if(!defined($entity)){ 
+		warn '$entity not defined';
+	}
 	# I don't like how this is called thrice.... but, oh well...
 	my $a_entity = undef;
 	
@@ -1135,15 +1225,24 @@ sub view_file_attachment {
 		-filename => $filename, 
 		-entity   => $entity
 	); 	
+	
+	# These are horrible.
 	if(! defined( $a_entity )){ 
+		
+		warn '$a_entity not defined.';
+		
 		$filename =~ s/ /+/g;		
 		$a_entity = $self->_find_filename_attachment_entity(
 			-filename => $filename, 
 			-entity   => $entity
 		);		
 	}
+	
 	# We sort of undo what we just did! 
 	if(! defined( $a_entity )){ 
+		
+		warn '$a_entity still not defined.';
+		
 		$filename =~ s/\+/\%20/g;
 		$a_entity = $self->_find_filename_attachment_entity(
 			-filename => $filename, 
@@ -1151,24 +1250,36 @@ sub view_file_attachment {
 		);		
 	}
 	if(! defined( $a_entity )){ 
-		return $q->header('text/plain') . 'Error: Cannot view attachment!'; 
+		
+		warn '$a_entity yet again not defined.';
+		
+		return ({-type => 'text/plain'},  'Error: Cannot view attachment!'); 
 	}
 	else { 
-	my $body     = $a_entity->bodyhandle;
-	
-	if($args{-mode} eq 'inline'){ 
-		$r .= $q->header($a_entity->head->mime_type); 
-	}else{ 
-	
-			$r .=  "Content-disposition: attachement; filename=\"$filename\"\n";
-	   		$r .=  "Content-type: application/octet-stream\n\n";
 	
+		my $body =  $a_entity->bodyhandle->as_string; 
+
+		my $h = {}; 
+	    
+		if($args{-mode} eq 'inline'){ 
+			# ?
+			$h->{-type} = $a_entity->head->mime_type; 
+			
+			$a_entity->purge;
+	 		undef($a_entity); 
+			
+			return ($h, $body);
+		
+		}else{ 
+		    $h->{'-Content-disposition'} = 'attachment; filename="' . $filename . '"';
+	#	   	$h->{-type}                  = 'application/octet-stream';
+	        $h->{-type} = $a_entity->head->mime_type; 
 		}
 	
-		# encoded. Yes or no?
-		$r .=  $body->as_string; 
-	
-		return $r; 	
+		$a_entity->purge;
+ 		undef($a_entity); 
+
+		return ($h, $body);	
 	}
 
 }
@@ -1178,6 +1289,9 @@ sub view_file_attachment {
 
 sub _find_filename_attachment_entity { 
 
+	warn '_find_filename_attachment_entity'
+		if $t;
+	
 	my $self = shift; 
 	my %args = (-filename => undef, 
 				-entity   => undef, 
@@ -1186,7 +1300,8 @@ sub _find_filename_attachment_entity {
 				
 	my $entity    = $args{-entity}; 
 	my $filename  = $args{-filename};
-
+	warn '$filename:' . $filename if $t; 
+	
 
 	my @parts = $entity->parts; 
 	
@@ -1202,16 +1317,31 @@ sub _find_filename_attachment_entity {
 					if $s_entity; 
 			}
 	}else{ 
+		
+		warn q{$entity->head->mime_attr("content-type.name"):}            . $entity->head->mime_attr("content-type.name") if $t; ;
+		warn q{$entity->head->mime_attr("content-disposition.filename"):} . $entity->head->mime_attr("content-disposition.filename") if $t; ; 
+		
 		my $name = $entity->head->mime_attr("content-type.name") || 
 			       $entity->head->mime_attr("content-disposition.filename");
+				   
+	   $name = safely_decode($name);
 	
+		warn '$self->_decode_header($name):' . $self->_decode_header($name) if $t;  
+		
 		if($name){ 
-			if($name eq $filename ){ 
+			warn 'name is defined.' if $t; 
+			if($self->_decode_header($name) eq $filename ){ 
+				warn 'found a match, returning entity' if $t; 
 				return $entity; 
 			}
+			else { 
+ 				warn '"' . $self->_decode_header($name) . '" ne "' . $filename . '"' if $t; 
+			}
 		}
 	}
 	
+	$entity->purge;
+	undef($entity); 
 	# If we get here, it means we weren't able to find the attachment, sadly. 
 	return undef; 
 }
@@ -1242,38 +1372,28 @@ sub _rearrange_cid_img_tags {
 	$body_copy =~ s/\>/\>\n/g;
 	$body_copy =~ s/\</\n\</g;
 	
-	if($self->can_display_attachments){ 
-		my @lines = split("\n", $body_copy); 
-		for my $line(@lines){ 
-			
-			
-			if($line =~ m/\"(cid\:(.*?))\"/){ 
-				push(@cids, $1); 
-			}
+
+	my @lines = split("\n", $body_copy); 
+	for my $line(@lines){ 
+		
+		
+		if($line =~ m/\"(cid\:(.*?))\"/){ 
+			push(@cids, $1); 
 		}
-		
-	
-		for my $this_cid(@cids){
-		
-		
-			my $img_url = $DADA::Config::PROGRAM_URL . '?f=show_img&l=' . $self->{list} . '&id=' . $args{-key} . '&cid=' . $this_cid;  
-	
-			my $link_wo_cid = $img_url; 
-			   $link_wo_cid =~ s/cid\://; 
-			
-				
-			my $qm_this_cid = quotemeta($this_cid); 
-			
-			$body =~ s/$qm_this_cid/$link_wo_cid/g;
-		}
-		
-	 	return $body; 
-	 	
-	}else{ 
-	
-		$body =~ s/\<img(.*?)(src=|src=\")(cid\:)(.*?)\>/<img$1$2$4>/g; #basically, just get rid of 'em and call it a day.
-		return $body; 	
 	}
+	
+
+	for my $this_cid(@cids){
+	
+		my $img_url = $DADA::Config::PROGRAM_URL . '/show_img/' . $self->{list} . '/' . $args{-key} . '/' . $this_cid;  
+		my $link_wo_cid = $img_url; 
+		   $link_wo_cid =~ s/cid\://; 
+		my $qm_this_cid = quotemeta($this_cid); 
+		$body =~ s/$qm_this_cid/$link_wo_cid/g;
+	}
+	
+ 	return $body; 
+
 
 }
 
@@ -1303,8 +1423,10 @@ sub view_inline_attachment {
 	
 	my $body; 
 	
-	my $a_entity = $self->_find_inline_attachment_entity(-cid => $args{-cid}, -entity => $entity); 
-	
+	my $a_entity = $self->_find_inline_attachment_entity(
+	    -cid    => $args{-cid}, 
+	    -entity => $entity
+	); 
 	
 	require CGI; 
 	my $q = CGI->new; 
@@ -1323,26 +1445,27 @@ sub view_inline_attachment {
 	
 	require MIME::Base64; 
 	
-	my $r; 
+	my $r; 	
+	my $h = {}; 
 	
 	if($c_type =~ m/image\/gif/){ 
-		$r .= $q->header('image/gif');
+		$h->{-type} = 'image/gif';
 	}elsif($c_type =~ m/image\/jpg|image\/jpeg/){ 
-		$r .= $q->header('image/jpg');
+		$h->{-type} = 'image/jpg';
 	}elsif($c_type =~ m/image\/png/){ 
-		$r .= $q->header('image/png');
+		$h->{-type} = 'image/png';
 	}elsif($c_type =~ m/application\/octet\-stream/){ # dude, this could be anything...
 		if($a_entity->head->mime_attr("content-type.name") =~ m/\.png$/i){
-			$r .= $q->header('image/png'); 
+		    $h->{-type} = 'image/png'; 
 		}elsif($a_entity->head->mime_attr("content-type.name") =~ m/\.jpg$|\.jpeg/i){
-			$r .= $q->header('image/jpg'); 
+			$h->{-type} = 'image/jpg'; 
 		}elsif($a_entity->head->mime_attr("content-type.name") =~ m/\.gif$/i){
-			$r .= $q->header('image/gif'); 
+			$h->{-type} = 'image/gif'; 
 		}
 	}else{ 
 		warn "unsupported content type! " .  $c_type; 
 
-		$r .= $q->header('image/png');
+		$h->{-type} = 'image/png';
 		# a simple, 1px png image. 
 		my $str = <<EOF
 iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAMAAAAoyzS7AAAABGdBTUEAANbY1E9YMgAAABl0RVh0
@@ -1350,15 +1473,18 @@ U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAAGUExURf///wAAAFXC034AAAABdFJOUwBA
 5thmAAAADElEQVR42mJgAAgwAAACAAFPbVnhAAAAAElFTkSuQmCC
 EOF
 ;
-		$r .= MIME::Base64::decode_base64($str);
-		return $r; 
-		
+		$r = MIME::Base64::decode_base64($str);
 	}
 	
 	   #Encoded. Yes or no?
-	   $r .=  $body->as_string; 
+	   
+ 
+	   $r =  $body->as_string; 
 	  
-	   return $r; 	
+	   $entity->purge;
+	   undef($entity); 
+		
+	   return ($h, $r); 	
 }
 
 
@@ -1461,9 +1587,12 @@ sub make_search_summary {
 		
 		if (! $message){ 
 			$message = $raw_msg;
-			$message = $self->massaged_msg_for_display(-key        => $key, 
-													   -plain_text => 1,
-													 );
+			$message = $self->massaged_msg_for_display(
+			    { 
+			        -key        => $key, 
+					-plain_text => 1,
+				}
+			);
 		}else{ 
 			$message = html_to_plaintext({-str => $message}); 
 		}
@@ -1473,7 +1602,7 @@ sub make_search_summary {
 		
 		for $line(@message_lines){ 
 			if($line =~ m/$keyword/io){ 
-				$line =~ s{$keyword}{<em class="highlighted">$keyword</em>}gi;
+				$line =~ s{$keyword}{<em class="dm_highlighted">$keyword</em>}gi;
 				$line = $self->massage($line); 
 				$search_summary{$key} .= "... $line ... <br />";
 			}
@@ -1513,12 +1642,22 @@ sub _faked_oldstyle_message {
 	
 	if(!$entity){
 		warn "Something's wrong $!"; 
+		
+ 	   $entity->purge;
+ 	   undef($entity); 
+		
 		return ('', ''); 	
 	}else{ 
 		$entity = $self->_get_body_entity($entity); 
-		
+			
 		# UnEncoded. YES. 
-		return ($entity->bodyhandle->as_string, $entity->head->mime_type); 
+		my $bas = $entity->bodyhandle->as_string;
+		my $mt  = $entity->head->mime_type;
+ 	   
+	   $entity->purge;
+ 	   undef($entity); 
+		
+		return ($bas, $mt); 
 	}
 }
 
@@ -1554,8 +1693,10 @@ sub message_blurb {
 		if !$args{-key}; 
 		
 	my $msg = $self->massaged_msg_for_display(
-		-key        => $args{-key}, 
-		-plain_text => 1,
+	    {
+    		-key        => $args{-key}, 
+    		-plain_text => 1,
+    	}	
 	);
 					
 	# We'll want to, actually, escape out the entities - I don't know
@@ -1571,9 +1712,9 @@ sub message_blurb {
 	my $take = $l < $size ? $l : $size; 
 	
 	# xss_filter is one way to do this, the other is
-	# going through the _scrub_js
+	# going through the scrub_js
 	# xss_filter is a LOT faster, and should do the job...
-	# anyways, it probably already is going through _scrub_js
+	# anyways, it probably already is going through scrub_js
 	# if it's also going through: massaged_msg_for_display
 	#return substr($msg, 0, $take); 
 	return xss_filter(substr($msg, 0, $take));
@@ -1599,6 +1740,7 @@ sub massage_msg_for_resending {
 	my $self = shift; 
 	my %args = (-key        => undef, 
 				'-split'     => 0,  
+				-zap_sigs    => 1,
 				@_); 
 
 	my ($subject, $message, $format, $raw_msg) = $self->get_archive_info($args{-key}, 1);
@@ -1611,8 +1753,10 @@ sub massage_msg_for_resending {
 	
 	
 	my $entity = $self->_entity_from_raw_msg($raw_msg); 
-	   $entity = $self->_take_off_sigs($entity); 
-	
+	   
+	if($args{-zap_sigs} == 1){
+		$entity = $self->_take_off_sigs($entity); 
+	}
 	# These may be out of date, so let's get rid of them.
 	for my $header(
 		'From', 
@@ -1632,13 +1776,23 @@ sub massage_msg_for_resending {
 	if($args{'-split'} == 1){ 
 		# Not sure about this one - probably want it unencoded, so that we can resend it? Meh?
 		
+		my $has = $entity->head->as_string;
+		my $bas = $entity->body_as_string; 
+		
+ 	   $entity->purge;
+  	   undef($entity); 
+		
 		return (
-			 safely_decode($entity->head->as_string),
-		   	 safely_decode($entity->body_as_string), 
+			 safely_decode($has),
+		   	 safely_decode($bas), 
 		) ;
 	}else{
 		my $str =  safely_decode($entity->as_string);
 		$str = $self->massage($str); 
+		
+  	   $entity->purge;
+   	   undef($entity); 
+		
 		return $str; 
 	}
 }
@@ -1676,6 +1830,10 @@ sub _take_off_sigs {
 			
 			if($content){ 
 				if($entity->head->mime_type eq 'text/html'){ 
+					
+					$content = $self->string_between_markers($content);
+					
+					
 					$content = $self->_zap_sig_html($content);
 				}else{ 
 					$content = $self->_zap_sig_plaintext($content);
@@ -1734,22 +1892,26 @@ document.
 sub massaged_msg_for_display {
 
     my $self = shift;
-
-    my %args = (
-        -key            => undef,
-        -body_only      => 0,
-        -plain_text     => 0,
-        -entity_protect => 1,
-
-        @_
-    );
+    my ($args) = @_;
+    
+    if(!exists($args->{-body_only})){ 
+        $args->{-body_only} = 0;         
+    } 
+    if(!exists($args->{-plain_text})){ 
+        $args->{-plain_text} = 0;         
+    } 
+    if(!exists($args->{-entity_protect})){ 
+        $args->{-entity_protect} = 1;         
+    } 
 
     my $content_type = 'text/html';
 
     my ( $subject, $message, $format, $raw_msg ) =
-      $self->get_archive_info( $args{-key} );
+      $self->get_archive_info( $args->{-key} );
 
     if ( !$raw_msg ) {
+		warn 'no raw message.'
+			if $t; 
         $raw_msg = $self->_bs_raw_msg( $subject, $message, $format );
     }
 
@@ -1759,7 +1921,7 @@ sub massaged_msg_for_display {
 	);
 	
     if ( !$entity ) {
-        carp "Couldn't create entity: " . $@;
+        carp "Couldn't create entity";
     }
 
     my $body;
@@ -1777,6 +1939,9 @@ sub massaged_msg_for_display {
         || $b_entity->head->mime_type eq 'text' )
     {
 
+		warn 'PlainText body' 
+			if $t; 
+		
         # If you want the I<unencoded> body, and you are dealing with a
         # singlepart message (like a "text/plain"), use C<bodyhandle()> instead:
 
@@ -1784,81 +1949,93 @@ sub massaged_msg_for_display {
 		$body = safely_decode($body);
 		 
 		
-        if ( $self->{ls}->param('stop_message_at_sig') == 1 ) {
-            $body = $self->_zap_sig_plaintext($body);
-        }
+        $body = $self->_zap_sig_plaintext($body);
 
         $body = $self->massage($body);
         $body = $self->_parse_in_list_info(
             -data => $body,
             (
-                  ( $args{-plain_text} == 1 )
+                  ( $args->{-plain_text} == 1 )
                 ? ( -type => 'text/plain' )
                 : ( -type => 'text/html' )
             ),
         );
 
-        if ( $args{-plain_text} == 1 ) {
+        if ( $args->{-plain_text} == 1 ) {
 
             # ...
         }
         else {
-			$body = plaintext_to_html({-str => $body});
+			$body = markdown_to_html({-str => $body});
         }
 
         if ( $self->{ls}->param('style_quoted_archive_text') == 1 ) {
             $body = $self->_highlight_quoted_text($body)
-              unless $args{-plain_text} == 1;
+              unless $args->{-plain_text} == 1;
         }
-        $content_type = 'text/plain';
+        $content_type = 'text/plain'; 		
 
     }
     elsif ( $b_entity->head->mime_type eq 'text/html' ) {
-
+		
+		warn 'HTML body' 
+			if $t; 
+		
         $body = $b_entity->bodyhandle->as_string;
 		$body = safely_decode($body); 
 		
+		
         $body = $self->_rearrange_cid_img_tags(
-            -key  => $args{-key},
+            -key  => $args->{-key},
             -body => $body,
         );		
-        if ( $self->{ls}->param('stop_message_at_sig') == 1 ) {
-            $body = $self->_zap_sig_html($body);
-        }
+			$body = $self->string_between_markers($body);			
+            
+			# This is more backwards compatible stuff:
+			$body = $self->_zap_sig_html($body);
+        
 
         $body = $self->massage($body);
+		
         $body = $self->_parse_in_list_info(
             -data => $body,
             (
-                  ( $args{-plain_text} == 1 )
+                  ( $args->{-plain_text} == 1 )
                 ? ( -type => 'text/plain' )
                 : ( -type => 'text/html' )
             ),
         );
+		
     }
     else {
         warn "I don't know how to work with what I have! mime_type: "
           . $b_entity->head->mime_type
           . ', key: '
-          . $args{-key}
+          . $args->{-key}
           . ', list: '
           . $self->{list_info}->{list};
+		  
     }
 
 	$body = $self->_neuter_confirmation_token_links($body);
 
-    $body = $self->_scrub_js($body)
+    $body = scrub_js($body)
       if $self->{ls}->param('disable_archive_js') == 1;
 
-    $body = $self->_email_protect( $b_entity, $body )
-      if $args{-entity_protect};
+	if ($args->{-entity_protect} == 1) {
+		$body = $self->_email_protect( $b_entity, $body )
+	}
+	
+    if($b_entity) {
+		$b_entity->purge;
+	    undef($b_entity); 		
+	}
 
-    if ( $args{-body_only} == 1 ) {
+    if ( $args->{-body_only} == 1 ) {
         $body = $self->_chomp_off_body($body);
     }
     else {
-        if ( $args{-plain_text} == 1 ) {
-
+        if ( $args->{-plain_text} == 1 ) {
             # ...
         }
         else {
@@ -1866,7 +2043,8 @@ sub massaged_msg_for_display {
         }
     }
 
-    if ( $args{-plain_text} == 1 ) {
+  	
+    if ( $args->{-plain_text} == 1 ) {
 		# happens when you have a HTML body and need it back in plaintext
 		# From what I can figure out, this'll only happen in the 
 		# message blurbs?
@@ -1881,7 +2059,10 @@ sub massaged_msg_for_display {
 		$body       =~ s/$closing/\]/g; 
 		
     }
-
+	
+   $entity->purge;
+   undef($entity); 
+		
     return wantarray ? ( $body, $content_type ) : $body;
 }
 
@@ -1905,7 +2086,6 @@ sub _parse_in_list_info {
             -list_settings_vars       => $self->{ls}->params,
             -list_settings_vars_param => {-dot_it => 1},
 			-subscriber_vars_param    => {-list => $self->{list}, -use_fallback_vars => 1},
-            -dada_pseudo_tag_filter   => 1, 
 
         }, 
 
@@ -1965,23 +2145,30 @@ sub _chomp_off_body {
 	
 	my $self  = shift; 
 	my $str   = shift;
-	my $n_str = $str;
 	
-	# code below replaces code above - any problems?
-	# yeah, it doesn't fucking work.
+	#warn '$str:' . $str; 
+	
+	
+	my $n_str = $str;
 	
 	if($n_str =~ m/\<body.*?\>|<\/body\>/i){ 
 
 		$n_str =~ m/\<body.*?\>([\s\S]*?)\<\/body\>/i;  
 		$n_str = $1; 
-		
 		if($n_str =~ m/\<body.*?\>|<\/body\>/i){ 
 			$n_str = $self->_chomp_off_body_thats_being_difficult($n_str); 		
-		}		
+		}	
+		$n_str = strip($n_str);
+		if($n_str =~ m/\<body.*?\>/) { #seriously?
+		
+			$n_str =~ s/\<body/\<x\-body/g; 
+		
+			$n_str =~ s/\<\/body/\<\/x-body/g; 
+		}
 	}
-		
+	
+	
 	if(!$n_str){
-		
 		return $str; 
 	}else{ 
 		return $n_str;
@@ -2116,7 +2303,10 @@ sub _get_body_entity {
 		# singlepart. 
 		if(($entity->head->mime_type eq 'text/html') || ($entity->head->mime_type eq 'text/plain')){ 
 			#ha! done. 
-				return $entity; 
+			if(!defined($entity)) {
+				carp 'did not find a single part HTML or PlainText body entity.';
+			}
+			return $entity; 
 		} 
 	}
 }
@@ -2346,9 +2536,12 @@ sub atom_index {
 
 		my ($subject, $message, $format) = $self->get_archive_info($entries->[$i]);
 		
-		$message = $self->massaged_msg_for_display(-key       => $entries->[$i], 
-										           -body_only => 1
-										         );	
+		$message = $self->massaged_msg_for_display(
+		    { 
+		        -key       => $entries->[$i], 
+				-body_only => 1,
+			}
+		);
         
         $message = HTML::Entities::Numbered::name2decimal($message); 
         
@@ -2469,11 +2662,12 @@ sub rss_index {
 		
 		my ($subject, $message, $format, $raw_msg) = $self->get_archive_info($entries->[$i]);
 		
-			$message = $self->massaged_msg_for_display(-key       => $entries->[$i], 
-												       -body_only => 1
-											          );	
-											          
-			
+			$message = $self->massaged_msg_for_display(
+			    {
+			        -key       => $entries->[$i], 
+					-body_only => 1
+				}
+			);	
 			   
 			$message =~ s/&/\&amp;/g;  
 			$message =~ s/>/\&gt;/g;
@@ -2537,14 +2731,20 @@ http://www.xmlrpc.com/weblogsCom
 sub send_pings {
 
 	my $self = shift; 
+	my $can_use_xmlrpc_lite = 1; 
 	
 	if($self->{ls}->param('show_archives')        && 
 	   $self->{ls}->param('publish_archives_rss') && 
 	   $self->{ls}->param('ping_archives_rss')
 	  ){ 
 	
-		eval { require XMLRPC::Lite };
-		if(! $@ ) { 
+		try { 
+			require XMLRPC::Lite
+		} catch { 
+			$can_use_xmlrpc_lite = 0;  
+		};
+		
+		if($can_use_xmlrpc_lite == 1 ) { 
 			return map {
 				eval { XMLRPC::Lite->proxy($_, timeout => 5)
 				->call('weblogUpdates.ping', $self->{ls}->param('list_name') , $DADA::Config::PROGRAM_URL . '/list/' . $self->{list} . '/')
@@ -2597,33 +2797,26 @@ sub _decode_header {
 	my $self       = shift; 	 
 	my $header     = shift;
 	my $new_header = undef; 
+	my $can_use_mime_encodewords = 1; 
 	
-	if($self->{ls}->param('mime_encode_words_in_headers') == 1){ 
-		eval{ 
-			require MIME::EncWords;
-		};
-#
-		if($@){ 
-			carp "MIME::EncWords is returning with an error: $@"; 		
-			return $header;
-		}
-		else  {
+	try { 
+		require MIME::EncWords;
+	} catch {
+		$can_use_mime_encodewords = 0; 
+		carp 'MIME::EncWords is returning with an error:' . $_; 		
+		return $header;
+	};
 
+	if($can_use_mime_encodewords == 1){ 
+			$new_header = MIME::EncWords::decode_mimewords($header, Charset => '_UNICODE_'); 
+			return $new_header;
+	}		
+}
 
-			#if($header eq MIME::EncWords::decode_mimewords($header)){ 
-				# No? Well, nothing to do; 
-				#...
-			#}
-			#else { 
-				# Yes? Let's decode!
-				$new_header = MIME::EncWords::decode_mimewords($header, Charset => '_UNICODE_'); 
-				return $new_header;
-		}
-		
-	}
-	else { 
-		return $header; 
-	}
+sub DESTROY { 
+	#my $self = shift; 
+    #$self->{parser}->filer->purge
+	#	if $self->{parser};
 }
 
 
@@ -2636,7 +2829,7 @@ sub _decode_header {
 
 =head1 COPYRIGHT 
 
-Copyright (c) 1999 - 2014 Justin Simoni All rights reserved. 
+Copyright (c) 1999 - 2017 Justin Simoni All rights reserved. 
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
