@@ -2,15 +2,21 @@ package DADA::MailingList::Settings;
 use strict;
 use lib qw(./ ../ ../../ ../../DADA ../perllib); 
 
+
+my $t = 0; 
+
 use Carp qw(croak carp); 
 
 my $type; 
 my $backend; 
 use DADA::Config qw(!:DEFAULT); 	
 use DADA::App::Guts; 
+use DADA::Template::Widgets; 
 
 BEGIN { 
+    
 	$type = $DADA::Config::SETTINGS_DB_TYPE;
+	
 	if($type eq 'SQL'){ 
 	 	if ($DADA::Config::SQL_PARAMS{dbtype} eq 'mysql'){ 
 			$backend = 'MySQL';
@@ -40,19 +46,12 @@ sub _init  {
     if($args->{-new_list} == 1){ 
 	
 		$self->{name} = $args->{-list};
-		# $self->{local_li} = $self->get;
-		# warn 'in init: $args->{-list} ' . $args->{-list}; 	
 	}else{ 
 		
 		if($self->_list_name_check($args->{-list}) == 0) { 
     		croak('BAD List name "' . $args->{-list} . '" ' . $!);
 		}
-		else { 
-			#$self->{local_li} = $self->get; 
-		}		
-		
 	}
-
 }
 
 
@@ -78,9 +77,6 @@ sub get {
         while (my ($k, $v) = each(%$ls)){
             $new_ls->{'list_settings.' . $k} = $v; 
         }
-        # Will $ls undef every ref to it to?!
-        #undef($ls); 
-        
         return $new_ls; 
 
 	}
@@ -99,6 +95,11 @@ sub post_process_get {
     my $li   = shift;
     my $args = shift;
 
+    if(! exists($args->{-all_settings})){ 
+        $args->{-all_settings} = 0; 
+    }
+   #  warn '$args->{-all_settings} ' . $args->{-all_settings}; 
+    
     carp "$DADA::Config::PROGRAM_NAME $DADA::Config::VER warning! List "
       . $self->{function}
       . " db empty!  List setting DB Possibly corrupted!"
@@ -143,7 +144,7 @@ sub post_process_get {
                 last;
             }
         }
-
+        
         for ( 'sasl_smtp_password', 'pop3_password',
             'discussion_pop_password' )
         {
@@ -172,21 +173,34 @@ sub post_process_get {
 
         for ( keys %$li ) {
             if ( exists( $li->{$_} ) ) {
-
                 if ( !defined( $li->{$_} ) ) {
-
                     delete( $li->{$_} );
                 }
             }
         }
 
-        for ( keys %DADA::Config::LIST_SETUP_DEFAULTS ) {
-
-            if ( !exists( $li->{$_} ) || length( $li->{$_} ) == 0 ) {
-                $li->{$_} = $DADA::Config::LIST_SETUP_DEFAULTS{$_};
+        if($args->{-all_settings} == 1) { 
+            my $start_time = time; 
+            my $html_settings          = $self->_html_settings;
+            
+            for ( keys %DADA::Config::LIST_SETUP_DEFAULTS ) {
+                if ( !exists( $li->{$_} ) || length( $li->{$_} ) == 0 ) {
+                 	if(exists($html_settings->{$_})) {
+                        $li->{$_} = $self->_fill_in_html_settings($_);                      
+                    }
+                    else { 
+                        $li->{$_} = $DADA::Config::LIST_SETUP_DEFAULTS{$_};
+                    }
+                }
+            }
+        }
+        else { 
+            for ( keys %DADA::Config::LIST_SETUP_DEFAULTS ) {
+                if ( !exists( $li->{$_} ) || length( $li->{$_} ) == 0 ) {
+                    $li->{$_} = $DADA::Config::LIST_SETUP_DEFAULTS{$_};               
+                }
             }
         }
-
 		# This says basically, make sure the list subscription quota is <= the global list sub quota. 
         $DADA::Config::SUBSCRIPTION_QUOTA ||= undef;
 
@@ -235,10 +249,10 @@ sub post_process_get {
 		# ... 
     }
 	else { 
-	    $li->{enable_closed_loop_opt_in}            = 1;
-        $li->{enable_mass_subscribe}                = 0;
-		$li->{allow_admin_to_subscribe_blacklisted} = 0; 
-		$li->{skip_sub_confirm_if_logged_in}        = 0;	
+	    $li->{enable_closed_loop_opt_in}               = 1;
+        $li->{enable_mass_subscribe}                   = 0;
+        $li->{enable_mass_subscribe_only_w_root_login} = 0; 
+		$li->{allow_admin_to_subscribe_blacklisted}    = 0; 
 	}
     return $li;
 
@@ -251,14 +265,14 @@ sub params {
 	
 	my $self = shift; 
 	
-	if(keys %{$self->{local_li}}){ 
+	if(keys %{$self->{cached_settings}}){ 
 		#... 
 	}
 	else { 
-		$self->{local_li} = $self->get; 
+		$self->{cached_settings} = $self->get; 
 	}
 	
-	return $self->{local_li};
+	return $self->{cached_settings};
 	
 }
 
@@ -274,107 +288,92 @@ sub param {
 		croak "You MUST pass a name as the first argument!"; 
 	}
 	
-	if(keys %{$self->{local_li}}){ 
-		#warn "$name is cached, using cached stuff." ;
-		#... 
-	}
-	else { 
-		#warn "$name is NOT cached, fetching new stuff" ;
-		$self->{local_li} = $self->get; 
+	if(!exists($DADA::Config::LIST_SETUP_DEFAULTS{$name})){ 
+		croak "Cannot call param() on unknown setting, '$name'"; 
 	}
 	
-	if(defined($value)){ 
-		
-		if(!exists($DADA::Config::LIST_SETUP_DEFAULTS{$name})){ 
-			croak "Cannot call param() on unknown setting, '$name'"; 
+	
+	if(keys %{$self->{cached_settings}}){ 
+		warn "$name is cached, using cached stuff." if $t;  
+	}
+	else { 
+		warn "$name is NOT cached, fetching new stuff" if $t; 
+		$self->{cached_settings} = $self->get; 
+	}
+	
+	if(defined($value)){  
+		$self->save({-settings =>{$name => $value}});
+		$self->{cached_settings} = {};
+		return $value; # or... what should I return?
+	}
+	else { 
+	
+		if(exists($self->{cached_settings}->{$name}) && defined($self->{cached_settings}->{$name})) { 
+		    warn 'setting is cached and defined.' if $t; 
+			return $self->{cached_settings}->{$name};
+		}
+		elsif($self->_html_settings()->{$name}){ 
+		    warn 'setting isa _html_settings. ' if $t; 
+		    
+            if($self->{cached_settings}->{_cached_all_settings} == 1) {
+                warn 'all settings are cached, but the saved value seems to be blank!' if $t;  
+                # Guess it's... blank. 
+                return ''; 
+		    }
+		    else { 
+		        warn 'removing cache' if $t; 
+                $self->{cached_settings} = {}; 
+                warn 'creating cache, with all vals' if $t;     
+                $self->{cached_settings} = $self->get(-all_settings => 1); 
+                warn 'setting that cache has all vals' if $t; 
+                $self->{_cached_all_settings} = 1; 
+                warn 'returning val for, ' . $name if $t; 
+                return $self->{cached_settings}->{$name}; 
+		    }
+		}
+		elsif(! exists($self->{cached_settings}->{$name}) ) { 
+		    carp "Cannot fill in value for, '$name'";
+			return undef; 
 		}
 		else { 
-				
-			$self->save({$name => $value});
-			$self->{local_li} = {};
-			return $value; # or... what should I return?
+		    return ''; 
 		}
 	}
-	else { 
-	
-		# Why wasn't this here before?
-		#
-		if(!exists($DADA::Config::LIST_SETUP_DEFAULTS{$name})){ 
-			croak "Cannot call param() on unknown setting, '$name'"; 
-		}
-	
-		return $self->{local_li}->{$name};
-		
-	}
-	
-	
 }
 
-sub x_message_body_content { 
-	my $self = shift; 
-	my $type = shift || undef; 
-	die '$type cannot be undef!' 
-		if $type eq undef;  
-	my $param_name = ''; 
-	if($type =~ m/html/i){ 
-		$param_name = 'default_html_message_content'
-	}
-	else { 
-		$param_name = 'default_plaintext_message_content'; 
-	}
-	
-	my $str = '';
-	if($self->param($param_name . '_src') eq 'url_or_path'){ 
-		if(isa_url($self->param($param_name . '_src_url_or_path'))){ 
-			grab_url($self->param($param_name . '_src_url_or_path'));
-		}
-		elsif(-e $self->param($param_name . '_src_url_or_path')){ 
-			my $fn = make_safer($self->param($param_name . '_src_url_or_path')); 
-			my $d = undef; 
-			eval { $d  = slurp($fn); }; 
-			if($@){ 
-				carp $@;
-				return '';  
-			}
-			else { 
-				return $d; 
-			}
-		}
-		else { # 'default'
-			return ''; 
-		}
-	}
-	else { 
-		my $fn; 
-		if($type =~ m/html/i){ 
-			$fn = $DADA::Config::TEMPLATES . '/' . $param_name . '-' . $self->{name} . '.html'; 
-		}
-		else { 
-			$fn = $DADA::Config::TEMPLATES . '/' . $param_name . '-' . $self->{name} . '.txt'; 
-		}
-		$fn = make_safer($fn);  
-		if(-e $fn){
-			eval { $str = slurp($fn); }; 
-			if($@){ 
-				carp $@;
-				return '';  
-			}
-			else { 
-				return $str; 
-			}
-		} 
-		return $str
-	}
+sub _html_settings {
+    return {
+        html_confirmation_message         => 1,
+        html_subscribed_message           => 1,
+        html_unsubscribed_message         => 1,
+        html_subscription_request_message => 1,
+    };
+}
 
+sub _fill_in_html_settings { 
+    my $self = shift;
+	my $name = shift; 
+	
+	my $message_settings = { 
+        html_confirmation_message         => 'confirmation.tmpl',
+        html_subscription_request_message => 'subscription_request.tmpl',	    
+        html_subscribed_message           => 'subscribed.tmpl',
+        html_unsubscribed_message         => 'unsubscribed.tmpl',
+	}; 
+	
+    if(exists($message_settings->{$name})) { 
+        my $raw_screen = DADA::Template::Widgets::_raw_screen( 
+			{ 
+				-screen => 'list/' . $message_settings->{$name} 
+			} 
+		);
+        return $raw_screen; 
+	}
+	else { 
+		return undef; 
+	}
 }
-sub plaintext_message_body_content { 
-	my $self = shift; 
-	return $self->x_message_body_content('plaintext');
-}
-sub html_message_body_content { 
-	my $self = shift; 
-	return $self->x_message_body_content('html');
-}
+
 
 
 sub save_w_params {
@@ -383,6 +382,9 @@ sub save_w_params {
     my ($args)    = @_;
     my $associate = undef;
     my $settings  = {};
+	
+	# use Data::Dumper; 
+	# warn Dumper($args); 
 
     if ( !exists( $args->{-associate} ) ) {
         croak(
@@ -425,10 +427,31 @@ sub save_w_params {
 #	use Data::Dumper; 
 #	croak Data::Dumper::Dumper($saved_settings); 
 
-    return $self->save($saved_settings);
+	delete($args->{-associate});
+	$args->{-settings} = $saved_settings; 
+    return $self->save($args);
 	
 }
 
+sub also_save_for_list { 
+	my $self = shift; 
+	my $q    = shift;
+	
+	my $also_save_for      = $q->param("also_save_for") // undef; 
+	my @also_save_for_list = ();
+	if($also_save_for) {
+		@also_save_for_list = split(',', $q->param("also_save_for_list"));
+	}
+	if(scalar @also_save_for_list > 0){ 
+		return [@also_save_for_list];
+	}
+	else { 
+		return []; 
+	}
+	
+}
+
+
 
 
 
@@ -437,20 +460,11 @@ sub _existence_check {
 
     my $self = shift; 
     my $li   = shift; 
-    
     for(keys %$li){ 
-        #next if $_ eq 'list';
-        
-        if(!exists($DADA::Config::LIST_SETUP_DEFAULTS{$_})){ 
-        
-            croak("Attempt to save a unregistered setting - $_"); 
-        
+        if(!exists($DADA::Config::LIST_SETUP_DEFAULTS{$_})){         
+            croak("Attempt to save a unregistered setting: '$_'"); 
         }
-           
     }
-
-
-
 }
 
 
@@ -501,6 +515,43 @@ sub _trim {
 
 
 
+sub _dd_freeze {
+    my $self = shift;
+    my $data = shift;
+
+    require Data::Dumper;
+    my $d = new Data::Dumper( [$data], ["D"] );
+    $d->Indent(0);
+    $d->Purity(1);
+    $d->Useqq(0);
+    $d->Deepcopy(0);
+    $d->Quotekeys(1);
+    $d->Terse(0);
+
+    # ;$D added to make certain we get our data structure back when we thaw
+    return $d->Dump() . ';$D';
+
+}
+
+sub _dd_thaw {
+
+    my $self = shift;
+    my $data = shift;
+
+    # To make -T happy
+    my ($safe_string) =  $data =~ m/^(.*)$/s;
+    $safe_string = 'my ' . $safe_string; 
+    my $rv = eval($safe_string);
+    if ($@) {
+        croak "couldn't thaw data! - $@\n" . $data;
+    }
+    return $rv;
+}
+
+
+
+
+
 
 
 1; 
@@ -528,11 +579,11 @@ DADA::MailingList::Subscribers - API for the Dada Mailing List Settings
  
  
 	# Save a setting
-	$ls->save(
-		{
+	$ls->save({
+		-settings => {
 			list_name => "my list", 
 		}
-	);
+	});
  
  # save a setting, from a CGI parameter, with a fallback variable: 
  $ls->save_w_params(
@@ -565,7 +616,7 @@ make up the, "what" of your Dada Mail list. The settings hold things like the na
 
 =head2 Mailing List Settings Model
 
-Settings are saved in a key/value pair, as originally, the backend for all this was a dn file - and still is, for the default backend. This module basically manipulates that key/value hash. Very simple. 
+Settings are saved in a key/value pair, as originally, the backend for all this was a dn file. This module basically manipulates that key/value hash. Very simple. 
 
 =head2 Default Values of List Settings
 
@@ -609,7 +660,7 @@ None, really.
 
 =head2 save
 
- $ls->save({list_name => 'my new list name'}); 
+ $ls->save({-settings => {list_name => 'my new list name'}}); 
 
 C<save> accepts a hashref as a parameter. The hashref should contain key/value pairs of list settings you'd like to change. All key/values passed will re-write any options saved. There is no validation of the information you passed. 
 
@@ -701,7 +752,7 @@ Many different modules support passing parameter values to their own methods, as
 
 =head1 COPYRIGHT 
 
-Copyright (c) 1999 - 2014 Justin Simoni All rights reserved. 
+Copyright (c) 1999 - 2017 Justin Simoni All rights reserved. 
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
