@@ -13,7 +13,7 @@ my $email_id = $DADA::Config::SQL_PARAMS{id_column} || 'email_id';
 
 $DADA::Config::SQL_PARAMS{id_column} ||= 'email_id';
 
-my $t = $DADA::Config::DEBUG_TRACE->{DADA_MailingList_baseSQL};
+my $t = $DADA::Config::DEBUG_TRACE->{DADA_MailingList};
 
 use Fcntl qw(
   O_WRONLY
@@ -34,7 +34,6 @@ sub inexact_match {
     my ( $name, $domain ) = split ( '@', $email );
 
     my $query .= 'SELECT COUNT(*) ';
-
     $query .= ' FROM ' . $self->{sql_params}->{subscriber_table} . ' WHERE ';
     $query .= ' list_type = ? AND';
     $query .= ' list_status = ' . $self->{dbh}->quote(1);
@@ -47,7 +46,6 @@ sub inexact_match {
     else {
         $query .= ' AND list = ?';
     }
-    #$query .= ' AND (email = ? OR email LIKE ? OR email LIKE ?)';
 	$query .= ' AND (email = ? OR email = ? OR email = ?)';
 
     warn 'Query: ' . $query
@@ -58,15 +56,13 @@ sub inexact_match {
     if (   $args->{ -against } eq 'black_list'
         && $DADA::Config::GLOBAL_BLACK_LIST == 1 )
     {
-
-
 		$sth->execute(
 		    $args->{ -against },
 		    $email,
 		    $name . '@',
 		    '@' . $domain,
 		  )
-		  or croak "cannot do statment (inexact_match)! $DBI::errstr\n";
+		  or croak "cannot do statement (inexact_match)! $DBI::errstr\n";
 
     }
     else {
@@ -79,7 +75,7 @@ sub inexact_match {
 	    '@' . $domain,
 
 	  )
-	  or croak "cannot do statment (inexact_match)! $DBI::errstr\n";
+	  or croak "cannot do statement (inexact_match)! $DBI::errstr\n";
 	
     }
 
@@ -110,31 +106,45 @@ sub search_list {
     my $r = [];
 
     my $partial_listing = {};
-
+    my $search_type = 'any'; 
+    
     my $fields = $self->subscriber_fields;
-    for (@$fields) {
-        $partial_listing->{$_} = { like => $args->{-query} };
+    if(! exists($args->{-query}) && exists($args->{-partial_listing})){ 
+        $partial_listing = $args->{-partial_listing}; 
+        $search_type = 'all'; 
     }
-
-    # Do I have to do this, explicitly?
-    $partial_listing->{email} = { like => $args->{-query} };
-
+    else { 
+        for (@$fields) {
+            $partial_listing->{$_} = {
+                 -operator => 'LIKE',
+                 -value    => $args->{-query}, 
+            };
+        }
+        # Do I have to do this, explicitly?
+        $partial_listing->{email} = {
+            -operator => 'LIKE',
+            -value    => $args->{-query} 
+        };
+    }
+    
     my $query = $self->SQL_subscriber_profile_join_statement(
         {
             -type            => $args->{-type},
             -partial_listing => $partial_listing,
-            -search_type     => 'any',
+            -search_type     => $search_type,
             -order_by        => $args->{-order_by},
             -order_dir       => $args->{-order_dir},
 			#-start          => $args->{ -start }, 
 			#'-length'       => $args->{'-length'}, 
         }
     );
+    warn 'QUERY: ' . $query
+     if $t; 
 
     my $sth = $self->{dbh}->prepare($query);
 
     $sth->execute()
-      or croak "cannot do statment (for search_list)! $DBI::errstr\n";
+      or croak "cannot do statement (for search_list)! $DBI::errstr\n";
 
     my $row   = {};
     my $count = 0;
@@ -151,10 +161,16 @@ sub search_list {
         next if $count >  ( $args->{ -start } * $args->{ '-length' }) + ($args->{'-length'}) ;
 
         my $info = {};
-        $info->{email} = $row->{email};
-        $info->{type}  = $args->{-type};    # Whazza?!
+        
+        $info->{timestamp}            = $row->{timestamp};
+        $info->{email}                = $row->{email};
+		$info->{delivery_prefs_value} = $row->{delivery_prefs_value};
+        
+		
+		$info->{type}     = $args->{-type};    # Whazza?!
 
-        delete( $row->{email} );
+        delete( $row->{email} ); #?
+        
         $info->{fields} = [];
 
         for (@$fields) {
@@ -236,18 +252,19 @@ sub domain_stats {
 
 sub SQL_subscriber_profile_join_statement {
 
+    warn 'at, ' . ( caller(0) )[3] if $t;
+    
     my $self = shift;
     my ($args) = @_;
 
-    # Args
-    # -partial_listing
-    # -type
-    # -search_type (any/all)
-    # -list
-    #
-
+    if($t == 1){ 
+        require Data::Dumper; 
+        warn 'passed args:' . Data::Dumper::Dumper($args); 
+    }
     # init vars:
 
+    my $ls = DADA::MailingList::Settings->new({-list => $self->{list}}); 
+
     # type list black_List, white_listed, etc
     if ( !$args->{-type} ) {
         $args->{-type} = 'list';
@@ -259,27 +276,28 @@ sub SQL_subscriber_profile_join_statement {
     }
 
     if ( !exists( $args->{-order_by}) || !defined($args->{-order_by}) ) {
-        $args->{-order_by} = 'email';
+            $args->{-order_by} = 'email';
     }
-
     if ( !exists( $args->{-order_dir} ) ) {
         $args->{-order_dir} = 'asc';
     }
-
-
-
-    if ( exists( $args->{-include_from} ) ) {
-        if ( exists( $args->{-include_from}->[0] ) ) {
-
-            # ...
-        }
-        else {
-            delete( $args->{-include_from} );
-        }
+    if(!exists($args->{-user_order_by})){ 
+        $args->{-user_order_by} = 1; 
+    }
+    
+    if(!exists($args->{-mass_mailing_params})) { 
+        $args->{-mass_mailing_params} = 
+        {
+            -delivery_preferences => 'all', # individual, digest, all
+        }; 
     }
 
-# Right now, we can either have an any/all boolean type of thing. "OR" is used for
-# searches, I'm not sure if this would be helpful for the Partial List Sending stuff.
+	
+    if(!exists($args->{-show_delivery_prefs_column})){ 
+        $args->{-show_delivery_prefs_column} = 1; 
+    }
+	
+	
 
     my $query_type = 'AND';
     if ( !$args->{-search_type} ) {
@@ -291,35 +309,48 @@ sub SQL_subscriber_profile_join_statement {
     if ( $args->{-search_type} eq 'any' ) {
         $query_type = 'OR';
     }
-
-    my $subscriber_table     = $self->{sql_params}->{subscriber_table};
-    my $profile_fields_table = $self->{sql_params}->{profile_fields_table};
-
-    # This is to select which Subscriber Profile Fields to return with our query
-    my @merge_fields      = @{ $self->subscriber_fields };
-    my $merge_field_query = '';
-    for (@merge_fields) {
-        $merge_field_query .= ', ' . $profile_fields_table . '.' . $_;
+    
+    if(!exists($args->{-select_fields})){ 
+        $args->{-select_fields}->{':all'} = 1; 
     }
 
-   #/ This is to select which Subscriber Profile Fields to return with our query
-
+    my $subscriber_table       = $self->{sql_params}->{subscriber_table};
+    my $profile_fields_table   = $self->{sql_params}->{profile_fields_table};
+    my $profile_settings_table = $self->{sql_params}->{profile_settings_table};
     # We need the email and list from $subscriber_table
+	
+	# I'm not sure if "DISTINCT" is needed, anymore. 
     my $query;
+	   $query = 'SELECT DISTINCT ';
 
-    if ( exists( $args->{-include_from} )
-        && $self->{sql_params}->{dbtype} eq 'Pg' )
-    {
-        $query .= ' SELECT DISTINCT ON(' . $subscriber_table . '.email) ';
+    # This is to select which Profile Fields to return with our query
+    my @merge_fields = (); 
+    if($args->{-select_fields}->{'subscriber.timestamp'} == 1 || $args->{-select_fields}->{':all'} == 1){ 
+        push(@merge_fields, $subscriber_table . '.timestamp');
     }
-    else {
-        $query = 'SELECT ';
+    if($args->{-select_fields}->{'subscriber.email'} == 1 || $args->{-select_fields}->{':all'} == 1){
+        push(@merge_fields, $subscriber_table . '.email'); 
     }
+    if($args->{-select_fields}->{'subscriber.list'} == 1 || $args->{-select_fields}->{':all'} == 1){ 
+        push(@merge_fields, $subscriber_table . '.list'); 
+    }
+    foreach(@{ $self->subscriber_fields }) { 
+        if($args->{-select_fields}->{'profile.' . $_} == 1 || $args->{-select_fields}->{':all'} == 1){
+            push(@merge_fields, $profile_fields_table . '.' . $_)
+        }
+    }
+	
+	if($args->{-show_delivery_prefs_column} == 1) {
+		# Delivery Prefs
+	    if($ls->param('digest_enable') == 1){ 
+			push(@merge_fields, $profile_settings_table . '.value as delivery_prefs_value');
+		}
+	}
+	
+    $query .= join(', ', @merge_fields); 
 
-    $query .= $subscriber_table . '.email, ' . $subscriber_table . '.list';
-    $query .= $merge_field_query;
 
-# And we need to match this with the info in $profile_fields_table - this fast/slow?
+	# And we need to match this with the info in $profile_fields_table - this fast/slow?
     $query .=
         ' FROM '
       . $subscriber_table
@@ -329,8 +360,16 @@ sub SQL_subscriber_profile_join_statement {
       . $subscriber_table
       . '.email' . ' = '
       . $profile_fields_table
-      . '.email';
+      . '.email';	
 
+
+      if($ls->param('digest_enable') == 1){ 
+	  	$query .= ' LEFT JOIN ' 
+		. $profile_settings_table . ' ON (' 
+		. $subscriber_table .'.email = ' . $profile_settings_table . '.email AND '
+		. $subscriber_table .'.list = ' . $profile_settings_table . '.list) ';
+	}
+	
     # Global Black List spans across all lists (yes, we're still using this).
     $query .= ' WHERE  ';
     if (   $DADA::Config::GLOBAL_BLACK_LIST
@@ -340,24 +379,10 @@ sub SQL_subscriber_profile_join_statement {
         #... Nothin'
     }
     else {
-
-        if ( exists( $args->{-include_from} ) ) {
-            my @include_from = ( $self->{list}, @{ $args->{-include_from} } );
-            @include_from = map( $self->{dbh}->quote($_), @include_from );
-            @include_from =
-              map( $_ = $subscriber_table . '.list = ' . $_, @include_from );
-
-            my $include_from_query = join( ' OR ', @include_from );
-            $include_from_query = '( ' . $include_from_query . ' )';
-            $include_from_query .= ' AND ';
-            $query .= $include_from_query;
-        }
-        else {
-            $query .=
-                $subscriber_table
-              . '.list = '
-              . $self->{dbh}->quote( $self->{list} ) . ' AND ';
-        }
+	    $query .=
+	        $subscriber_table
+	      . '.list = '
+	      . $self->{dbh}->quote( $self->{list} ) . ' AND ';
     }
 
     # list_status is almost always 1
@@ -369,13 +394,14 @@ sub SQL_subscriber_profile_join_statement {
       . $subscriber_table
       . '.list_status = '
       . $self->{dbh}->quote('1') . ' ';
-
+      
     # This is all to query the $dada_profile_fields_table
     # The main thing, is that we only want the SQL statement to hold
     # fields that we're actually looking for.
 
     if ( keys %{ $args->{-partial_listing} } ) {
 
+        warn q|keys %{ $args->{-partial_listing} }| if $t; 
         # This *really* needs its own method, as well...
         # It's somewhat strange, as this relies on the email address in the
         # profile (I think?) to work, if we're looking for email addresses...
@@ -384,6 +410,9 @@ sub SQL_subscriber_profile_join_statement {
 
         for my $field( keys %{ $args->{-partial_listing} } ) {
 			
+			warn '$field ' . $field
+			 if $t; 
+			
 			my @s_snippets = ();
 			
             # This is to make sure we're always using the email from the
@@ -393,72 +422,82 @@ sub SQL_subscriber_profile_join_statement {
             if ( $field eq 'email' ) {
                 $table = $subscriber_table;
             }
+            elsif ($field eq 'subscriber.timestamp') { 
+                $table = $subscriber_table;             
+            }
 
-            my $search_type   = undef;
-            my $search_op     = undef;
-            my $search_pre    = undef;
-            my $search_app    = undef;
-			my $search_binder = undef; 
-            if ( exists( $args->{-partial_listing}->{$field}->{equal_to} ) ) {
-                if ( length( $args->{-partial_listing}->{$field}->{equal_to} ) > 0 )
-                {
-                    $search_type   = 'equal_to';
+            next if ! exists($args->{-partial_listing}->{$field}->{-value});
+            next if ! defined($args->{-partial_listing}->{$field}->{-value});
+            next if           $args->{-partial_listing}->{$field}->{-value} eq '';
+
+            
+            my $search_op        = '';
+            my $search_pre       = '';
+            my $search_app       = '';
+            my $search_binder    = '';
+
+            if($field ne 'subscriber.timestamp') { 
+                if ( $args->{-partial_listing}->{$field}->{-operator} eq '=' ) {
                     $search_op     = '=';
                     $search_pre    = '';
                     $search_app    = '';
                     $search_binder = 'OR';
-
                 }
-            }
-            elsif ( exists( $args->{-partial_listing}->{$field}->{like} ) ) {
-                if ( length( $args->{-partial_listing}->{$field}->{like} ) > 0 ) {
-                    $search_type   = 'like';
+                elsif ( $args->{-partial_listing}->{$field}->{-operator} eq 'LIKE' ) {
                     $search_op     = 'LIKE';
                     $search_pre    = '%';
                     $search_app    = '%';
                     $search_binder = 'OR';
-  
-              }
-            }
-            elsif ( exists( $args->{-partial_listing}->{$field}->{not_equal_to} ) ) {
-                if ( length( $args->{-partial_listing}->{$field}->{not_equal_to} ) > 0 ) {
-                    $search_type   = 'not_equal_to';
+                }
+                elsif ( $args->{-partial_listing}->{$field}->{-operator} eq '!=' ) {
                     $search_op     = '!=';
                     $search_pre    = '';
                     $search_app    = '';
                     $search_binder = 'AND';
-
                 }
-            }
-            elsif ( exists( $args->{-partial_listing}->{$field}->{not_like} ) ) {
-                if ( length( $args->{-partial_listing}->{$field}->{not_like} ) > 0 ) {
-                    $search_type   = 'not_like';
+                elsif ( $args->{-partial_listing}->{$field}->{-operator} eq 'NOT LIKE' ) {
                     $search_op     = 'NOT LIKE';
                     $search_pre    = '%';
                     $search_app    = '%';
                     $search_binder = 'AND';
-
                 }
-            }
 
-            next if $search_type eq undef;
-
-            my @terms = split(',', $args->{-partial_listing}->{$field}->{$search_type} );
-            foreach my $term(@terms) {
-				$term = strip($term); 
-                push(
-                    @s_snippets,
-                    $table . '.'
-                      . $field . ' '
-                      . $search_op . ' '
-                      . $self->{dbh}->quote(
-                            $search_pre
-                          . $term
-                          . $search_app
-                      )
-                );
-            }
-			 push( @add_q, '(' . join( ' ' . $search_binder . ' ', @s_snippets ) . ')' );
+                my @terms = split(',', $args->{-partial_listing}->{$field}->{-value} );
+                foreach my $term(@terms) {
+    				$term = strip($term); 
+                    push(
+                        @s_snippets,
+                        $table . '.'
+                          . $field . ' '
+                          . $search_op . ' '
+                          . $self->{dbh}->quote(
+                                $search_pre
+                              . $term
+                              . $search_app
+                          )
+                    );
+                }
+    			 push( @add_q, '(' . join( ' ' . $search_binder . ' ', @s_snippets ) . ')' );
+    		}
+    		elsif($field eq 'subscriber.timestamp') { 
+                my $timestamp_snippet = ''; 
+                
+                $timestamp_snippet = $table . '.timestamp >= ' 
+                . $self->{dbh}->quote(
+                        $args->{-partial_listing}->{$field}->{-rangestart}
+                    )
+                . ' AND '
+                . $table . '.timestamp <= ' 
+                . $self->{dbh}->quote(
+                    $args->{-partial_listing}->{$field}->{-rangeend}
+                ); 
+                warn 'pushing timestamp snippit: ' . $timestamp_snippet
+                    if $t;
+                push( @add_q, $timestamp_snippet );
+    		}
+    		else { 
+    		    # ... 
+    		}
         }
        
 
@@ -468,6 +507,10 @@ sub SQL_subscriber_profile_join_statement {
 	        $query .= $query_pl;
 	    }
 	}
+	else { 
+	    warn 'no -partial_listing' 
+	        if $t; 
+	}
 
    # -exclude_from is to return results from subscribers who *aren't* subscribed
    # to another list.
@@ -490,41 +533,95 @@ sub SQL_subscriber_profile_join_statement {
               . '.email = b.email) ';
             $query .= $ex_from_query;
         }
+    }    
+    
+
+#    require Data::Dumper; 
+#    warn '$args->{-mass_mailing_params}' . Data::Dumper::Dumper($args->{-mass_mailing_params}); 
+    
+    my $digest_subq = ' AND EXISTS (SELECT * FROM ' . 
+    $profile_settings_table . 
+    ' WHERE ' . 
+    $subscriber_table . 
+    '.email = ' . 
+    $profile_settings_table . 
+    '.email AND ' .
+    $profile_settings_table . 
+    '.list = ' . $self->{dbh}->quote( $self->{list}). 
+    ' AND ' . 
+    $profile_settings_table . 
+    '.setting = \'delivery_prefs\' AND ' .
+    $profile_settings_table . 
+    '.value = \'digest\') '; 
+    
+    my $individual_subq = ' AND NOT EXISTS (SELECT * FROM ' . 
+    $profile_settings_table . 
+    ' WHERE ' . 
+    $subscriber_table . 
+    '.email = ' . 
+    $profile_settings_table . 
+    '.email AND ' .
+    $profile_settings_table . 
+    '.list = ' . $self->{dbh}->quote( $self->{list}). 
+    ' AND ' . 
+    $profile_settings_table . 
+    '.setting = \'delivery_prefs\' AND (' .
+    $profile_settings_table . 
+    '.value = ' . $self->{dbh}->quote("digest") . ' OR ' . $profile_settings_table . 
+    '.value = ' . $self->{dbh}->quote("hold") .' ) ) ';
+        
+    if($args->{-mass_mailing_params}->{-delivery_preferences} eq 'digest'
+        && $ls->param('digest_enable') == 1
+    ){         
+        $query .= $digest_subq; 
+    }
+    elsif($args->{-mass_mailing_params}->{-delivery_preferences} eq 'individual'
+        && $ls->param('digest_enable') == 1
+    ){        
+        $query .= $individual_subq; 
     }
 
-    # /
+    if($args->{-user_order_by} == 1){ 
+        if ( $args->{-order_by} eq 'email' ) {
 
-    if ( exists( $args->{-include_from} )
-        && $self->{sql_params}->{dbtype} =~ m/^mysql$|^SQLite$/ )
-    {
-        $query .= ' GROUP BY ' . $subscriber_table . '.email ';
-    }
-
-    if ( $args->{-order_by} eq 'email' ) {
-
-        $query .=
+            $query .=
+                ' ORDER BY '
+              . $subscriber_table
+              . '.list, '
+              . $subscriber_table
+              . '.email';
+        }
+        elsif ( $args->{-order_by} eq 'timestamp' ) {
+            $query .=
+                ' ORDER BY '
+              . $subscriber_table
+              . '.list, '
+              . $subscriber_table
+              . '.timestamp';
+        }
+        elsif ( $args->{-order_by} eq 'delivery_prefs_value' ) {
+            $query .=
             ' ORDER BY '
           . $subscriber_table
           . '.list, '
-          . $subscriber_table
-          . '.email';
-    }
-    else {
-        $query .=
-            ' ORDER BY '
-          . $subscriber_table
-          . '.list, '
-          . $profile_fields_table . '.'
-          . $args->{-order_by};
+          . $profile_settings_table . '.value '
+		}
+        else {
+            $query .=
+                ' ORDER BY '
+              . $subscriber_table
+              . '.list, '
+              . $profile_fields_table . '.'
+              . $args->{-order_by};
 
-    }
-	if($args->{-order_dir} eq 'desc'){ 
-		$query .= ' DESC'; 
+        }
+    	if($args->{-order_dir} eq 'desc'){ 
+    		$query .= ' DESC'; 
+    	}
+    	else { 
+    		$query .= ' ASC'; 
+    	}
 	}
-	else { 
-		$query .= ' ASC'; 
-	}
-	
 	if(exists($args->{ -start }) && exists($args->{ '-length' })) { 
 		$query .= ' LIMIT '; 
 		$query .=  $args->{'-length'};
@@ -533,13 +630,84 @@ sub SQL_subscriber_profile_join_statement {
 	}
 	
     warn 'QUERY: ' . $query
-      if $t;
-
+             if $t;
+    
     return $query;
 }
 
 
-sub fancy_print_out_list {
+
+sub SQL_subscriber_update_profiles_statement {
+    
+    my $self = shift; 
+    my ($args) = @_; 
+
+    my $u = $args->{-update_fields};
+    
+    # This should help things not be crazy slow. 
+    $args->{-select_fields}->{'subscriber.email'} = 1; 
+    $args->{-user_order_by} = 0; 
+    $args->{-search_type}   = 'all'; 
+    
+    my $subscriber_table     = $self->{sql_params}->{subscriber_table};
+    my $profile_fields_table = $self->{sql_params}->{profile_fields_table};
+    
+    my $inner_query = $self->SQL_subscriber_profile_join_statement($args); 
+    
+    my $query = 'UPDATE ' . $profile_fields_table . '  SET '; 
+
+    foreach (keys %$u ){ 
+        $query .= $_ . ' = ' . $self->{dbh}->quote($u->{$_}) . ','; 
+    }
+    # Remove that last comma, we should do the above, with a join() 
+    # or a map() or something 
+    
+     
+    $query  =~ s/\,$//; 
+    $query .= ' WHERE email IN ( SELECT * FROM (';
+    $query .= $inner_query;
+    $query .= ') AS X)';     
+        
+    return $query; 
+    
+} 
+
+
+sub update_profiles { 
+    
+    my $self = shift; 
+    my ($args) = @_; 
+    my $query = $self->SQL_subscriber_update_profiles_statement($args); 
+    
+    # die 'query: ' . $query; 
+    
+    my $sth = $self->{dbh}->prepare($query);
+
+    my $rv = $sth->execute()
+      or croak "cannot do statement (for profile_update)! $DBI::errstr\n";
+    
+      return $rv; 
+
+}
+
+
+sub fancy_print_out_list { 
+    my $self = shift;
+    my ($args) = @_;
+    
+    if ( !exists( $args->{ -FH } ) ) {
+        $args->{ -FH } = \*STDOUT;
+    }
+    my $fh = $args->{ -FH };
+    
+    my($scrn, $subscribers_count) = $self->fancy_list($args); 
+    e_print($scrn, $fh); 	
+    
+}
+
+
+
+sub fancy_list {
 
     my $count = 0;
 
@@ -551,10 +719,6 @@ sub fancy_print_out_list {
 'you must supply the type of list we are looking at in, the "-type" parameter';
     }
 
-    if ( !exists( $args->{ -FH } ) ) {
-        $args->{ -FH } = \*STDOUT;
-    }
-    my $fh = $args->{ -FH };
 
     if ( !exists( $args->{ -partial_listing } ) ) {
         $args->{ -partial_listing } = {};
@@ -562,15 +726,20 @@ sub fancy_print_out_list {
 
 	my $show_list_column = 0; 
 	if(exists($args->{-show_list_column})){ 
-		$show_list_column = $args->{-show_list_column}; 
-		
+		$show_list_column = $args->{-show_list_column}; 	
 	}
+	my $show_timestamp_column = 0; 
+	if(exists($args->{-show_timestamp_column})){ 
+		$show_timestamp_column = $args->{-show_timestamp_column}; 	
+	}
+
 
     my $subscribers = $self->subscription_list($args);
     for (@$subscribers) {
-        $_->{no_email_links}   = 1;
-        $_->{no_checkboxes}    = 1;
-		$_->{show_list_column} = $show_list_column, 
+        $_->{no_email_links}        = 1;
+        $_->{no_checkboxes}         = 1;
+        $_->{show_timestamp_column} = $show_timestamp_column, 
+		$_->{show_list_column}      = $show_list_column, 
     }
 
     my $field_names = [];
@@ -583,19 +752,19 @@ sub fancy_print_out_list {
         {
             -screen => 'fancy_print_out_list_widget.tmpl',
             -vars   => {
-                field_names      => $field_names,
-                subscribers      => $subscribers,
-                no_checkboxes    => 1,
-                no_email_links   => 1,
-                show_list_column => $show_list_column, 
-                count            => scalar @{$subscribers},
+                field_names           => $field_names,
+                subscribers           => $subscribers,
+                no_checkboxes         => 1,
+                no_email_links        => 1,
+                show_list_column      => $show_list_column, 
+                show_timestamp_column => $show_timestamp_column, 
+                count                 => scalar @{$subscribers},
             }, 
 			-expr => 1, 
         }
     );
-	print $fh $scrn; 
-
-    return scalar @{$subscribers};
+     
+    return ($scrn, scalar @{$subscribers}); 
 
 }
 
@@ -603,7 +772,17 @@ sub print_out_list {
 
     my $self = shift;
 	my ($args) = @_; 
+
+    my $ls = DADA::MailingList::Settings->new({-list => $self->{list}}); 
 	
+#	use Data::Dumper; 
+#	warn 'args!' . Dumper($args); 
+	
+	my $r; 
+	
+	if(! exists($args->{-print_out})){ 
+	    $args->{-print_out} = 1;
+	}
 	if(! exists($args->{-fh})){ 
 			$args->{-fh} =  \*STDOUT;
 	}
@@ -614,49 +793,80 @@ sub print_out_list {
 	if(! exists($args->{-query})){ 
 		$args->{-query} = undef; 
 	}	
-#	if(! exists($args->{-order_by})){ 
-#		$args->{-order_by} = undef; 
-#	}	
-	if(! exists($args->{-order_dir})){ 
-		$args->{-order_dir} = undef; 
+
+	if(! exists($args->{-show_timestamp_column})){ 
+		$args->{-show_timestamp_column} = 1; 
+	}	
+
+	if(! exists($args->{-show_profile_fields})){ 
+		$args->{-show_profile_fields} = 1; 
+	}	
+		
+	if(! exists($args->{-show_delivery_prefs_column})){ 
+		$args->{-show_delivery_prefs_column} = 1; 
 	}
 	
 	# DEV: There's a reason for this tmp var, correct?
     my $fh = $args->{ -fh };
 
-	binmode $fh, ':encoding(' . $DADA::Config::HTML_CHARSET . ')';
+	#binmode $fh, ':encoding(' . $DADA::Config::HTML_CHARSET . ')';
 
     my $count;
 	my $query = ''; 
 
-	if(defined($args->{-query})){
-	 
-		my $partial_listing = {};
-
-	    my $fields = $self->subscriber_fields;
-	    for (@$fields) {
-	        $partial_listing->{$_} = { like => $args->{ -query } };
-	    }
-	    # Do I have to do this, explicitly?
-	    $partial_listing->{email} = { like => $args->{ -query } };
-	
-		$query = $self->SQL_subscriber_profile_join_statement(  
+    if(exists($args->{-partial_listing})){ 
+                
+        $query = $self->SQL_subscriber_profile_join_statement(  
 			{ 
-		    -partial_listing => $partial_listing,
-	        -search_type     => 'any',
-			-type            => $args->{-type},
-			-query           => $args->{-query},
-			-order_by        => $args->{-order_by},
-			-order_dir       => $args->{-order_dir},
+		    -partial_listing            => $args->{-partial_listing},
+	        -search_type                => 'all',
+			-type                       => $args->{-type},
+			-order_by                   => $args->{-order_by},
+			-order_dir                  => $args->{-order_dir},
+			-show_timestamp_column      => $args->{-show_timestamp_column}, 
+			-show_delivery_prefs_column => $args->{-show_delivery_prefs_column},
 			}
 		);  
+		
+    }
+	elsif(defined($args->{-query})){    
+		my $partial_listing = {};
+	    my $fields = $self->subscriber_fields;
+	    for (@$fields) {
+	        $partial_listing->{$_} = { 
+	            -operator => 'LIKE',
+                -value    => $args->{ -query },
+	            };
+	    }
+	    # Do I have to do this, explicitly?
+	    $partial_listing->{email} = { 
+	        -operator => 'LIKE',
+            -value    => $args->{ -query },
+	    };
+
+		$query = $self->SQL_subscriber_profile_join_statement(  
+			{ 
+		    -partial_listing            => $partial_listing,
+	        -search_type                => 'any',
+			-type                       => $args->{-type},
+			-query                      => $args->{-query},
+			-order_by                   => $args->{-order_by},
+			-order_dir                  => $args->{-order_dir},
+			-show_timestamp_column      => $args->{-show_timestamp_column}, 
+			-show_delivery_prefs_column => $args->{-show_delivery_prefs_column},
+			
+            
+			}
+		);
 	}
 	else { 
 	    $query =
 	      $self->SQL_subscriber_profile_join_statement(
 	        { 
-				-type      => $args->{-type}, 
-
+				-type                       => $args->{-type}, 
+				-show_timestamp_column      => $args->{-show_timestamp_column},
+				-show_delivery_prefs_column => $args->{-show_delivery_prefs_column},
+				
 			} 
 		);
 	}
@@ -664,24 +874,40 @@ sub print_out_list {
     my $sth = $self->{dbh}->prepare($query);
 
     $sth->execute()
-      or croak "cannot do statment (for print out list)! $DBI::errstr\n";
+      or croak "cannot do statement (for print out list)! $DBI::errstr\n";
 
     my $fields = $self->subscriber_fields;
 
+	# $DADA::Config::TEXT_CSV_PARAMS->{sep_char} = "\t";
     require Text::CSV;
-    my $csv = Text::CSV->new($DADA::Config::TEXT_CSV_PARAMS);
+    my $csv = Text::CSV->new(
+		$DADA::Config::TEXT_CSV_PARAMS, 
+	);
 
     my $hashref = {};
 
+
     my @header = ('email');
-    for (@$fields) {
-        push ( @header, $_ );
+	
+    if($args->{-show_timestamp_column} == 1){ 
+        push(@header, 'timestamp'); 
     }
-
+    
+    if($args->{-show_profile_fields} == 1){
+	    for (@$fields) {
+	        push ( @header, $_ );
+	    }
+	}
+	
+	if($args->{-show_delivery_prefs_column} == 1) {
+	    if($ls->param('digest_enable') == 1){ 
+			push(@header, 'delivery_prefs_value');
+		}
+	}
+	
     if ( $csv->combine(@header) ) {
-
         my $hstring = $csv->string;
-        print $fh $hstring, "\n";
+        $r .= $hstring . "\n";
 
     }
     else {
@@ -692,31 +918,40 @@ sub print_out_list {
     }
 
     while ( $hashref = $sth->fetchrow_hashref ) {
-
-        my @info = ( $hashref->{email} );
-
-        for (@$fields) {
-
-# DEV: Do we remove newlines here? Huh?
-# BUG: [ 2147102 ] 3.0.0 - "Open List in New Window" has unwanted linebreak?
-# https://sourceforge.net/tracker/index.php?func=detail&aid=2147102&group_id=13002&atid=113002
-            $hashref->{$_} =~ s/\n|\r/ /gi;
-            push ( @info, $hashref->{$_} );
-
+        
+		# Email!
+		my @info = ( $hashref->{email} );
+		
+		# Timestamp!
+        if($args->{-show_timestamp_column} == 1){ 
+            push(@info, $hashref->{timestamp}); 
         }
-
+		
+		# Profile Fields!
+		if($args->{-show_profile_fields} == 1){
+	        for (@$fields) {
+	            $hashref->{$_} =~ s/\n|\r/ /gi;
+	            push ( @info, $hashref->{$_} );
+	        }
+		}
+		
+		# Delivery Prefs!
+		if($args->{-show_delivery_prefs_column} == 1) {		
+		    if($ls->param('digest_enable') == 1){ 
+				push(@info, $hashref->{delivery_prefs_value});
+	        }
+		}
+		
         if ( $csv->combine(@info) ) {
             my $string = $csv->string;
-            print $fh $string, "\n";
+            $r .= $string . "\n";
         }
         else {
             my $err = $csv->error_input;
-
-            # carp "combine() failed on argument: ", $err, "\n";
-
             carp "combine() failed on argument: "
               . $csv->error_input
               . " attempting to encode values and try again...";
+              
             require CGI;
 
             my @new_info = ();
@@ -725,23 +960,25 @@ sub print_out_list {
             }
             if ( $csv->combine(@new_info) ) {
                 my $hstring2 = $csv->string;
-                print $fh $hstring2, "\n";
-               # carp "that worked.";
+                $r .= $hstring2 . "\n";
             }
             else {
                 carp "combine() failed on argument: "
                   . $csv->error_input;
-
             }
-
         }
-
         $count++;
     }
 
     $sth->finish;
-    return $count;
-
+    if($args->{-print_out} == 1){ 
+        print $fh $r;
+    }
+    else { 
+        return $r; 
+    }
+        
+    
 }
 
 sub clone {
@@ -783,9 +1020,14 @@ sub clone {
 
 
 sub subscription_list {
-
+        
     my $self = shift;
 
+    warn 'subscription_list'
+     if $t; 
+
+    my $st = time; 
+
     my ($args) = @_;
     if ( !exists( $args->{-start} ) ) {
         $args->{-start} = 0;
@@ -807,7 +1049,7 @@ sub subscription_list {
     my $sth   = $self->{dbh}->prepare($query);
 
     $sth->execute()
-      or croak "cannot do statment (for subscription_list)! $DBI::errstr\n";
+      or croak "cannot do statement (for subscription_list)! $DBI::errstr\n";
 
     my $hashref;
     my %mf_lt = ();
@@ -816,7 +1058,6 @@ sub subscription_list {
     }
 
     while ( $hashref = $sth->fetchrow_hashref ) {
-
         # Probably, just add it here?
         $hashref->{type} = $args->{-type};
 
@@ -841,6 +1082,11 @@ sub subscription_list {
 
     }
 
+    my $et = time; 
+    
+    warn 'subscription_list time:' . ($et - $st) . ' seconds.'
+        if $t;
+
     return $list;
 
 }
@@ -874,14 +1120,14 @@ sub filter_list_through_blacklist {
 
         $sth->execute()
           or croak
-          "cannot do statment (filter_list_through_blacklist)! $DBI::errstr\n";
+          "cannot do statement (filter_list_through_blacklist)! $DBI::errstr\n";
 
     }
     else {
 
         $sth->execute( $self->{list} )
           or croak
-          "cannot do statment (filter_list_through_blacklist)! $DBI::errstr\n";
+          "cannot do statement (filter_list_through_blacklist)! $DBI::errstr\n";
     }
 
     my $hashref;
@@ -902,7 +1148,7 @@ sub filter_list_through_blacklist {
         my $sth2 = $self->{dbh}->prepare($query2);
         $sth2->execute( $self->{list}, '%' . $hashref->{email} . '%' )
           or croak
-          "cannot do statment (filter_list_through_blacklist)! $DBI::errstr\n";
+          "cannot do statement (filter_list_through_blacklist)! $DBI::errstr\n";
 
         while ( $hashref2 = $sth2->fetchrow_hashref ) {
             push ( @$list, $hashref2 );
@@ -981,7 +1227,7 @@ sub check_for_double_email {
                 $args{ -Status }
               )
               or croak
-              "cannot do statment (for check for double email)! $DBI::errstr\n";
+              "cannot do statement (for check for double email)! $DBI::errstr\n";
             while ( ( my $email ) = $sth->fetchrow_array ) {
                 push ( @list, $email );
             }
@@ -1032,40 +1278,8 @@ $sth->execute();
 
 }
 
-sub remove_from_list {
 
-    my $self = shift;
 
-    carp
-"This method (DADA::MailingList::Subscribers::baseSQL::remove_from_list) is deprecated. Please use, remove_subscriber() instead.";
-
-    my %args = (
-        -Email_List => [],
-        -Type       => "list",
-        @_
-    );
-    my $addresses = $args{ -Email_List };
-
-    my $count = 0;
-    require DADA::MailingList::Subscriber;
-    for my $sub (@$addresses) {
-        chomp($sub);    #?
-        my $s = DADA::MailingList::Subscriber->new(
-            {
-                -list  => $self->{list},
-                -email => $sub,
-                -type  => $args{ -Type },
-            }
-        );
-
-        my $remove = $s->remove;
-
-        if ( $remove == 1 ) {
-            $count = $count + 1;
-        }
-    }
-    return $count;
-}
 
 sub remove_all_subscribers {
 
@@ -1158,15 +1372,16 @@ sub create_mass_sending_file {
     my $self = shift;
 
     my %args = (
-        -Type            => 'list',
-        -Pin             => 1,
-        -ID              => undef,
-        -Ban             => undef,
-        -Bulk_Test       => 0,
-        -Save_At         => undef,
-        -Test_Recipient  => undef,
-        -partial_sending => {},
-        -exclude_from    => [],
+        -Type                => 'list',
+        -Pin                 => 1,
+        -ID                  => undef,
+        -Ban                 => undef,
+        -Bulk_Test           => 0,
+        -Save_At             => undef,
+        -Test_Recipient      => undef,
+        -partial_sending     => {},
+        -mass_mailing_params => {},
+        -exclude_from        => [],
         @_
     );
 
@@ -1186,6 +1401,7 @@ sub create_mass_sending_file {
 
     $list =~ s/ /_/g;    # really...
 
+
     my ( $sec, $min, $hour, $day, $month, $year ) =
       (localtime)[ 0, 1, 2, 3, 4, 5 ];
     my $message_id = sprintf(
@@ -1234,22 +1450,34 @@ sub create_mass_sending_file {
 		$have_first_recipient = 0; 
 	}
 	# Sending these types of messages to the list owner is very confusing
-	if($type =~ m/_tmp\-just_subscribed\-|_tmp\-just_unsubscribed\-|_tmp\-just_subed_archive\-/){ 
+	elsif($type =~ m/_tmp\-just_subscribed\-|_tmp\-just_unsubscribed\-|_tmp\-just_subed_archive\-/){ 
 		$have_first_recipient = 0; 		
 	}
+	elsif($type =~ m/invitelist/){ 
+		$have_first_recipient = 0; 			
+	}
 
     require     Text::CSV;
     my $csv   = Text::CSV->new($DADA::Config::TEXT_CSV_PARAMS);
     my $total = 0;
 
+    warn '$args{-Test_Recipient} ' . $args{-Test_Recipient}
+            if $t; 
+    warn '$args{-Bulk_Test}' . $args{-Bulk_Test}
+        if $t; 
+            
 	if($have_first_recipient == 1){ 
 	    my $first_email = $self->{ls}->param('list_owner_email');
 	    if ( 
-			$args{'-Bulk_Test'} == 1 
+			$args{-Bulk_Test} == 1 
 		 && $args{-Test_Recipient} 
 		) {
 	        $first_email = $args{-Test_Recipient};
 	    }
+	    
+	    warn '$first_email ' . $first_email
+            if $t; 
+            
 	    my ( $lo_e_name, $lo_e_domain ) = split( '@', $first_email );
 
 	    my @lo  = (
@@ -1292,10 +1520,10 @@ sub create_mass_sending_file {
 		
         my $query = $self->SQL_subscriber_profile_join_statement(
             {
-                -type            => $args{-Type},
-                -partial_listing => $args{-partial_sending},
-                -exclude_from    => $args{-exclude_from},
-                -include_from    => $args{-include_from},
+                -type                => $args{-Type},
+                -partial_listing     => $args{-partial_sending},
+                -mass_mailing_params => $args{-mass_mailing_params},
+                -exclude_from        => $args{-exclude_from},
             }
         );
 
@@ -1378,17 +1606,19 @@ sub unique_and_duplicate {
     my $address_ref = $args{ -New_List };
 
     if ($address_ref) {
-
-        for (@$address_ref) { $lookup_table{$_} = 0 }
+        
+        for (@$address_ref) { 
+            $lookup_table{$_} = 0 
+        }
 
         my $email;
 
         my $sth = $self->{dbh}->prepare(
             "SELECT email FROM "
               . $self->{sql_params}->{subscriber_table}
-              . " WHERE list = ? 
-	                                      AND list_type = ?
-	                                      AND  list_status   = '1'"
+              . " WHERE list         = ? 
+                  AND list_type      = ?
+                  AND  list_status   = '1'"
         );
         $sth->execute( $self->{list}, $args{ -Type } )
           or croak
@@ -1396,7 +1626,6 @@ sub unique_and_duplicate {
         while ( ( my $email ) = $sth->fetchrow_array ) {
             chomp($email);
             $lookup_table{$email} = 1 if ( exists( $lookup_table{$email} ) );
-
             #nabbed it,
         }
         $sth->finish;
@@ -1492,7 +1721,7 @@ sub can_have_subscriber_fields {
 
 =head1 COPYRIGHT 
 
-Copyright (c) 1999 - 2014 Justin Simoni All rights reserved. 
+Copyright (c) 1999 - 2017 Justin Simoni All rights reserved. 
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
