@@ -115,14 +115,7 @@ sub _init {
 sub exists {
 	
     my $self = shift;
-  #  my ($args) = @_;
 
-	# This is saying, if we don't have a dbh handle, we don't have a proper 
-	# "handle" on a profile. 
-	
-	if(! exists($self->{dbh})){ 
-		return 0; 
-	}
     my $query =
       'SELECT COUNT(*) FROM '
       . $DADA::Config::SQL_PARAMS{profile_table}
@@ -135,16 +128,18 @@ sub exists {
 
     $sth->execute($self->{email} )
       or croak "cannot do statement (at exists)! $DBI::errstr\n";
-    my @row = $sth->fetchrow_array();
-    $sth->finish;
 
-	# autoviv?
-    if($row[0]){ 
-		return 1; 
-	}
-	else { 
-		return 0; 
-	}
+      my $count = $sth->fetchrow_array;
+
+    $sth->finish;
+    
+
+    if ( $count eq undef ) {
+        return 0;
+    }
+    else {
+        return $count;
+    }
 
 }
 
@@ -197,7 +192,7 @@ sub remove {
     my $sth = $self->{dbh}->prepare($query);
 
 	my $rv = $sth->execute( $self->{ email } )
-      or croak "cannot do statment (at remove)! $DBI::errstr\n";
+      or croak "cannot do statement (at remove)! $DBI::errstr\n";
 
     $sth->finish;
 
@@ -317,6 +312,17 @@ sub get {
 	if(!$self->exists({-email => $args->{-email}})){ 
 		return undef; 
 	}
+	
+	if(!exists($args->{-dotted})){ 
+		$args->{-dotted} = 0; 
+	}
+	if(!exists($args->{-dotted_with})){ 
+		$args->{-dotted_with} = 'profile'; 
+	} 
+			 
+	
+	
+	
 
     my $query =
       'SELECT * FROM '
@@ -351,7 +357,11 @@ sub get {
     if ( $args->{ -dotted } == 1 ) {
         my $dotted = {};
         for ( keys %$profile_info ) {
-            $dotted->{ 'profile.' . $_ } = $profile_info->{$_};
+            $dotted->{ 
+				$args->{-dotted_with} 
+				. '.' 
+				. $_ 
+			} = $profile_info->{$_};
         }
         return $dotted;
     }
@@ -379,7 +389,7 @@ sub remove {
     warn 'QUERY: ' . $query . ' (' . $self->{email} . ')'
       if $t;
     my $rv = $sth->execute( $self->{email} )
-      or croak "cannot do statment (at remove)! $DBI::errstr\n";
+      or croak "cannot do statement (at remove)! $DBI::errstr\n";
     $sth->finish;
     return $rv;
 }
@@ -465,6 +475,7 @@ sub subscribed_to {
 			                  authorized_senders => 'Authorized Senders',
 			                  moderators         => 'Moderators',
 			                  sub_request_list   => 'Subscription Requests',
+#			                  unsub_request_list => 'Unsubscription Requests',
 							  bounced_list       => 'Bouncing Addresses',
 			);
 			
@@ -580,9 +591,17 @@ sub profile_update_email_report {
                     'over_subscription_quota',
                     'already_sent_sub_confirmation',
                     'no_list',
+                    'profile_fields',
+					'stop_forum_spam_check_failed', 
+					'suspicious_activity_by_ip_check_failed',
+					'captcha_challenge_failed',
                 ],
             }
         );
+        # warn 'email: ' . $info->{update_email}; 
+        # warn 'status: ' . $sub_status;
+        # warn 'errors:' . Data::Dumper::Dumper($sub_errors); 
+        
         require DADA::MailingList::Settings;
         my $ls = DADA::MailingList::Settings->new( { -list => $list } );
         push (
@@ -600,37 +619,63 @@ sub profile_update_email_report {
 	
 }
 
-sub update_email { 
-	my $self = shift; 
-	my $old_fields = $self->{fields}->get;
-	my $info = $self->get; 
-	# Probably some check here, just to be thorough...
+
+
+
+sub update_email {
+
+    my $self       = shift;
+	my ($args)     = @_; 
+    my $old_fields = $self->{fields}->get;
+    my $info       = $self->get;
+
+    # Probably some check here, just to be thorough...
+
+    # This updates the profile
+    $self->update(
+        {
+            -activated              => 1,
+            -email                  => $info->{update_email},
+            -update_email           => '',
+            -update_email_auth_code => '',
+        }
+    );
+
+    # THis updates its fields
+    my $new_prof_fields = DADA::Profile::Fields->new( { -email => $info->{update_email} } );
+    $new_prof_fields->insert(
+        {
+            -fields    => $old_fields,
+            -confirmed => 1,
+            -mode      => 'writeover',
+        }
+    );
+
+    # Why isn't the old profile fields stuff not removed?
+    my $old_prof_fields = DADA::Profile::Fields->new( { -email => $info->{email} } );
+    $old_prof_fields->remove;
+    undef($old_prof_fields);
+    
+    # This updates the DADA::Profile::Settings; 
+	if(exists($args->{-list})){
+	    require DADA::Profile::Settings;
+	    my $dps = DADA::Profile::Settings->new(
+			{
+				-list => $args->{-list}
+			}
+		);
+		$dps->update(
+			{ 
+				-from => $self->{email},
+				-to   => $info->{update_email},
+			}
+		); 
+	}
 	
-	# This updates the profile 
-	$self->update(
-		{
-			-activated 			   => 1, 
-			-email		           => $info->{update_email}, 
-			-update_email           => '', 
-			-update_email_auth_code => '', 			
-		}	
-	); 
-	
-	# THis updates its fields
-	my $new_prof_fields = DADA::Profile::Fields->new({-email => $info->{update_email}}); 
-	$new_prof_fields->insert(
-		{
-			-fields    => $old_fields,
-			-confirmed => 1, 
-			-mode      => 'writeover', 
-		}
-	); 
-	# Why isn't the old profile fields stuff not removed? 
-	my $old_prof_fields = DADA::Profile::Fields->new({-email => $info->{email}}); 
-	   $old_prof_fields->remove;
-		undef($old_prof_fields); 
-	$self->{email} = $info->{update_email};
-	return 1; 
+    $self->{email} = $info->{update_email};
+    
+    
+    return 1;
 }
 
 
@@ -648,16 +693,18 @@ sub is_activated {
     warn 'QUERY: ' . $query
 		if $t; 
 
-    $sth->execute( $args->{ -email } )
+    $sth->execute( $self->{email}  )
       or croak "cannot do statement (is_activated)! $DBI::errstr\n";
  #   my @row = $sth->fetchrow_array();
 
     my $activated = 0;
   	FETCH: while ( my $hashref = $sth->fetchrow_hashref ) {
-        $activated = $hashref->{activated};
-        last FETCH;
+  	    
+  	    if($hashref->{activated} == 1) { 
+            $activated = $hashref->{activated};
+            last FETCH;
+        }
     }
-
     $sth->finish;
     return $activated;
 }
@@ -688,7 +735,7 @@ sub activate {
       if $t;
 
     my $rv = $sth->execute( $args->{ -activate }, $self->{email} )
-      or croak "cannot do statment (at activate)! $DBI::errstr\n";
+      or croak "cannot do statement (at activate)! $DBI::errstr\n";
     $sth->finish;
     return 1;
 }
@@ -824,37 +871,37 @@ sub is_valid_registration {
     }
 
     my $cap             = undef;
-	my $can_use_captcha = 1; 
 
 	if($DADA::Config::PROFILE_OPTIONS->{enable_captcha} == 1){
-		try { 
-			require DADA::Security::AuthenCAPTCHA; 
-		} catch {
-			carp "CAPTCHA Not working correctly?: $_";  
-			$can_use_captcha = 0;
-		};	
-	    if ( $can_use_captcha == 1 ) {
-	        $cap = DADA::Security::AuthenCAPTCHA->new;
-	        my $result = $cap->check_answer(
-	            $DADA::Config::RECAPTCHA_PARAMS->{private_key},
-	            $DADA::Config::RECAPTCHA_PARAMS->{'remote_address'},
-	            $args->{ -recaptcha_challenge_field },
-	            $args->{ -recaptcha_response_field },
-	        );
-	        if ( $result->{is_valid} == 1 ) {
-
-	            # ...
-	        }
-	        else {
+	    if ( can_use_Google_reCAPTCHA() == 1 ) {
+									
+			my $crf = $args->{ -recaptcha_response_field }  || undef;
+		   
+		    if ( ! $crf ) {
 	            $errors->{captcha_failed} = 1;
 	            $status = 0;
-	        }
+		    }
+		    else {				
+		        $cap = DADA::Security::AuthenCAPTCHA::Google_reCAPTCHA->new;
+		        my $result = $cap->check_answer(
+					$ENV{'REMOTE_ADDR'}, 
+					$crf,
+		        );
+		        if ( $result->{is_valid} == 1 ) {
+		            # ...
+		        }
+		        else {
+		            $errors->{captcha_failed} = 1;
+		            $status = 0;
+		        }
+			}
 	    }
 	}
     return ( $status, $errors );
 
 }
 
+
 sub is_valid_update_profile_email { 
 	
 	my $self = shift; 
@@ -896,22 +943,23 @@ sub is_valid_update_profile_activation {
 		invalid_auth_code => 0, 
   	}; 
 
-
-	if($self->exists){ 
+	if($self->exists == 1){ 
 		# ... 
 	}
 	else { 
         $errors->{profile_no_exists} = 1;
         $status = 0;		
+        return ($status, $errors);
 	}
-	my $new_prof = DADA::Profile->new({-email => $args->{ -updated_email }}); 
-    
-    if ( $new_prof->exists ) {
+	
+	my $profile = $self->get;
+	
+	my $new_prof = DADA::Profile->new({-email => $profile->{ update_email }}); 
+    if ( $new_prof->exists == 1 ) {
         $errors->{profile_exists} = 1;
         $status = 0;
     }
-
-	my $profile = $self->get;
+    undef($new_prof); 
 
 	if ( check_for_valid_email( $profile->{ update_email } ) == 0 ) {
         # ...
@@ -959,11 +1007,11 @@ sub update {
     }
 
     $self->remove();
-	
     $self->insert($new);
 
 }
 
+
 sub setup_profile {
     my $self = shift;
     my ($args) = @_;
@@ -997,29 +1045,44 @@ sub send_profile_activation_email {
       . '/'
       . $auth_code 
       . '/';
-
-    require DADA::App::Messages;
-    DADA::App::Messages::send_generic_email(
+	
+	
+  	require DADA::App::EmailThemes; 
+  	my $em = DADA::App::EmailThemes->new(
+  		{ 
+  			-list      => $self->_config_profile_host_list,
+  		}
+  	);
+	my $etp = $em->fetch('profiles_activation_message'); 
+	
+	require DADA::App::Messages;
+	my $dap = DADA::App::Messages->new(
+		{
+			-list => $self->_config_profile_host_list,
+		}
+	);
+	
+	$dap->send_multipart_email(		 
         {
-            -email   => $self->{email},
             -headers => {
-                Subject => $DADA::Config::PROFILE_ACTIVATION_MESSAGE_SUBJECT,
+			    Subject => $etp->{vars}->{subject},
                 From    => $self->_config_profile_email(1),
                 To      => $self->{email},
             },
-            -body        => $DADA::Config::PROFILE_ACTIVATION_MESSAGE,
-            -tmpl_params => {
-                -vars => {
-                    auth_code                    => $auth_code,
-                    email                        => $self->{email},
-                    'profile.email'              => $self->{email},
-                    'profile.email_name'         => $n,
-                    'profile.email_domain'       => $d,
-                    'app.profile_activation_link' => $profile_activation_link, 
-                },
-            },
-        }
-    );
+			-plaintext_body => $etp->{plaintext},
+			-html_body      => $etp->{html},
+	            -tmpl_params => {
+	                -vars => {
+	                    auth_code                     => $auth_code,
+	                    email                         => $self->{email},
+	                    'profile.email'               => $self->{email},
+	                    'profile.email_name'          => $n,
+	                    'profile.email_domain'        => $d,
+	                    'app.profile_activation_link' => $profile_activation_link, 
+	                },
+	            },
+	        }
+	    );
 
     return 1;
 
@@ -1041,17 +1104,29 @@ sub send_profile_reset_password_email {
     . '/'
     . $auth_code 
     . '/';
+    
+	
+	require DADA::App::EmailThemes; 
+	my $em = DADA::App::EmailThemes->new(
+		{ 
+			-list      => $self->_config_profile_host_list,
+			
+			
+		}
+	);
+	my $etp = $em->fetch('profiles_reset_password_message');
+	
     require DADA::App::Messages;
-    DADA::App::Messages::send_generic_email(
+    my $dap = DADA::App::Messages->new({-list => $self->_config_profile_host_list});
+	$dap->send_multipart_email(
         {
             -email   => $self->{email},
             -headers => {
-                Subject =>
-                  $DADA::Config::PROFILE_RESET_PASSWORD_MESSAGE_SUBJECT,
-                From => $self->_config_profile_email(1),
-                To   => $self->{email},
+                Subject => $etp->{vars}->{subject}, 
+                From    => $self->_config_profile_email(1),
+                To      => $self->{email},
             },
-            -body        => $DADA::Config::PROFILE_RESET_PASSWORD_MESSAGE,
+            -plaintext_body  => $etp->{plaintext},
             -tmpl_params => {
                 -vars => {
                     auth_code                             => $auth_code,
@@ -1122,17 +1197,26 @@ sub send_update_profile_email_email {
 	
 	my $info = $self->get({-dotted => 1}); 
 	
+	require DADA::App::EmailThemes; 
+	my $em = DADA::App::EmailThemes->new(
+		{ 
+			-list      => $self->_config_profile_host_list,
+			
+			
+		}
+	);
+	my $etp = $em->fetch('profiles_update_email_message');
+
 	require DADA::App::Messages;
-    DADA::App::Messages::send_generic_email(
+    my $dap = DADA::App::Messages->new({-list => $self->_config_profile_host_list});
+	$dap->send_multipart_email(
         {
-            -email   => $args->{-updated_email},
             -headers => {
-                Subject =>
-                  $DADA::Config::PROFILE_UPDATE_EMAIL_MESSAGE_SUBJECT,
+                Subject => $etp->{vars}->{subject},
                 From => $self->_config_profile_email(1),
                 To   => $args->{-updated_email},
             },
-            -body        => $DADA::Config::PROFILE_UPDATE_EMAIL_MESSAGE,
+            -plaintext_body => $etp->{plaintext},
             -tmpl_params => {
                 -vars => {
                     'profile.update_email_auth_code' => $args->{-update_email_auth_code},
@@ -1169,15 +1253,26 @@ sub send_update_email_notification {
 	}
 	my $info = $self->get({-dotted => 1}); 
 	
+	require DADA::App::EmailThemes; 
+	my $em = DADA::App::EmailThemes->new(
+		{ 
+			-list      => $self->_config_profile_host_list,
+			
+			
+		}
+	);
+	my $etp = $em->fetch('profiles_email_updated_notification_message');
+	
 	require DADA::App::Messages;
-    DADA::App::Messages::send_generic_email(
+    my $dap = DADA::App::Messages->new({-list => $self->_config_profile_host_list});
+	$dap->send_multipart_email(
         {
             -headers => {
-                Subject => $DADA::Config::PROFILE_EMAIL_UPDATED_NOTIFICATION_MESSAGE_SUBJECT,
+                Subject => $etp->{vars}->{subject},
                 From => $self->_config_profile_email(1),
                 To   => $self->_config_profile_email,
             },
-            -body        => $DADA::Config::PROFILE_EMAIL_UPDATED_NOTIFICATION_MESSAGE,,
+            -plaintext_body => $etp->{plaintext},
             -tmpl_params => {
                 -vars => {
 					'profile.prev_email'             => $args->{-prev_email}, 
@@ -1232,7 +1327,7 @@ sub set_auth_code {
         warn 'QUERY: ' . $query
           if $t;
         my $rv = $sth->execute( $auth_code, $self->{email} )
-          or croak "cannot do statment (at set_auth_code)! $DBI::errstr\n";
+          or croak "cannot do statement (at set_auth_code)! $DBI::errstr\n";
         $sth->finish;
         return $auth_code;
     }
@@ -1256,11 +1351,9 @@ sub _config_profile_email {
     my $self = shift;
 	my $n    = shift || undef; 
  
-	
     if ( length($DADA::Config::PROFILE_OPTIONS->{profile_email}) > 0) {
 		my @good_addresses = (); 
-		require Email::Address;
-		my @addrs = Email::Address->parse( $DADA::Config::PROFILE_OPTIONS->{profile_email} );
+		my @addrs = Email_Address_parse( $DADA::Config::PROFILE_OPTIONS->{profile_email} );
 
 		for my $a(@addrs) { 
 			if(DADA::App::Guts::check_for_valid_email($a->address) == 0){ 
@@ -1295,15 +1388,47 @@ sub _config_profile_email {
     }
 }
 
+sub _config_profile_host_list { 
+	
+    my $self      = shift; 
+    my $host_list = undef; 
+	
+	if ( length($DADA::Config::PROFILE_OPTIONS->{profile_host_list}) > 0) {
+        if(check_if_list_exists(-List => $DADA::Config::PROFILE_OPTIONS->{profile_host_list}) == 1){ 
+            return $DADA::Config::PROFILE_OPTIONS->{profile_host_list};
+        }
+        else { 
+            warn 'list, ' . $DADA::Config::PROFILE_OPTIONS->{profile_host_list} . ' does not exist.'; 
+           # return undef; 
+        }
+    }
+    else { 
+        #return undef; 
+    }
+	if(! defined($host_list)) { 
+        require DADA::App::Guts;
+        my @l = DADA::App::Guts::available_lists();
+		return $l[0];
+	}
+}
+
 sub _magic_config_profile_email { 
 	my $self = shift; 
 	
-    # magically.
-    require DADA::App::Guts;
-    my @l = DADA::App::Guts::available_lists();
-    require DADA::MailingList::Settings;
-    my $ls = DADA::MailingList::Settings->new( { -list => $l[0] } );
-    return $ls->param('list_owner_email');
+	if(defined($self->_config_profile_host_list)){ 
+        require DADA::MailingList::Settings;
+	    my $ls = DADA::MailingList::Settings->new( { -list => $self->_config_profile_host_list } );
+        return $ls->param('list_owner_email');
+	}
+	else { 
+	    
+        # magically.
+        require DADA::App::Guts;
+        my @l = DADA::App::Guts::available_lists();
+        require DADA::MailingList::Settings;
+        my $ls = DADA::MailingList::Settings->new( { -list => $l[0] } );
+        return $ls->param('list_owner_email');
+    }
 }
 
 sub _is_integer {
@@ -1314,7 +1439,8 @@ sub _is_integer {
 
 sub feature_enabled { 	
 	#my $self = shift; 
-	my $feature || undef; 
+	my $feature = shift || undef; 
+	
 	my $enabled = $DADA::Config::PROFILE_OPTIONS->{features}; 
 
 
@@ -1607,7 +1733,7 @@ If CAPTCHA is enabled for Profiles, (via the Config.pm C<$PROFILE_OPTIONS-E<gt>{
 
 =back 
 
-C<-recaptcha_challenge_field> and C<-recaptcha_response_field> map to the 3rd and 4th arguments you have to pass in C<DADA::Security::AuthenCAPTCHA>'s method, C<check_answer>, which is sadly currently under documented, but 
+C<-recaptcha_challenge_field> and C<-recaptcha_response_field> map to the 3rd and 4th arguments you have to pass in C<require DADA::Security::AuthenCAPTCHA::Google_reCAPTCHA;>'s method, C<check_answer>, which is sadly currently under documented, but 
 follows the same API as Captcha::reCAPTCHA: 
 
 L<http://search.cpan.org/~andya/Captcha-reCAPTCHA/lib/Captcha/reCAPTCHA.pm>
@@ -1701,9 +1827,9 @@ This method will croak if the profile you're trying to set an authorization code
 
 Justin Simoni http://dadamailproject.com
 
-=head1 LICENCE AND COPYRIGHT
+=head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 1999 - 2014 Justin Simoni All rights reserved. 
+Copyright (c) 1999 - 2017 Justin Simoni All rights reserved. 
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
