@@ -22,7 +22,7 @@ use warnings;
 our $VERSION = '1.00';
 use base 'Exporter';
 our @EXPORT = qw();
-use Switch;
+#use Switch;
 use Digest::SHA qw (hmac_sha1_base64 hmac_sha256_base64 sha256);
 use URI::Escape qw (uri_escape_utf8);
 use LWP 6;
@@ -34,7 +34,6 @@ use XML::LibXML;
 
 #use Time::HiRes qw(gettimeofday);
 
-my $endpoint          = 'https://email.us-east-1.amazonaws.com/';
 my $service_version   = '2010-12-01';
 my $tools_version     = '1.1';
 my $signature_version = 'HTTP';
@@ -51,6 +50,7 @@ my %allowed = (
     creds          => '',
 	AWSAccessKeyId => undef, 
 	AWSSecretKey   => undef, 
+	AWS_endpoint   => 'https://email.us-east-1.amazonaws.com/', 
     trace          => 0,
     encode_utf8    => 1,
 );
@@ -74,6 +74,10 @@ sub new {
 
 sub AUTOLOAD {
     my $self = shift;
+    
+	# don't do any work if we are being called for DESTROY
+    return if(substr($AUTOLOAD, -7) eq 'DESTROY');
+	
     my $type = ref($self)
       or croak "$self is not an object";
 
@@ -107,6 +111,12 @@ sub _init {
 	if(exists($args->{AWSSecretKey})) { 
 		$self->AWSSecretKey($args->{AWSSecretKey}); 
 	}
+	
+	if(exists($args->{AWS_endpoint})) { 
+		$self->AWS_endpoint($args->{AWS_endpoint}); 
+	}
+
+
     $self->creds( $args->{-creds} );
 }
 
@@ -124,12 +134,125 @@ sub reset_browser {
 
 }
 
+
+=cut 
+
+useless. 
+
+sub list_identities {  
+    my $self   = shift; 
+	my ($args) = @_; 
+	
+	my $params = {
+	    Action       => 'ListIdentities', 
+	};
+	my ($response_code, $response_content) = $self->call_ses($params, {});
+
+        use Data::Dumper; 
+        print '$response_content ' . Dumper($response_content); 
+        
+	if ( $self->trace ) {
+		print $response_code . "\n"; 
+        print $response_content . "\n"; 
+	}
+
+	if($response_code eq '200') { 
+		#if ( $self->trace ) {
+		#	$self->print_response($response_content);
+    	#}    
+	#	return ($response_code, $self->get_response($response_content));
+#	    return ($response_code, $response_content); 
+        my $r; 
+        my $parser = XML::LibXML->new();
+        my $dom = $parser->parse_string($response_content);
+        my $xpath = XML::LibXML::XPathContext->new($dom);
+        $xpath->registerNs('ns', $aws_email_ns);
+
+        my @nodes = $xpath->findnodes('/ns:ListIdentitiesResponse' . '/ns:ListIdentitiesResult'  . '/ns:Identities');
+        my @a = (); 
+        foreach my $node (@nodes) {
+            my $text = $node->textContent();
+        	push(@a, $text);
+        }
+		return ($response_code, \@a);
+	}
+	else { 
+		return ($response_code, $response_content);
+	}
+}
+
+=cut
+
+sub sender_verified { 
+
+    my $self  = shift; 
+    my $email = shift; 
+    
+    my ($name, $domain) = split('@', $email, 2); 
+	
+	# This is very limited and contrived
+	# See: https://metacpan.org/pod/Domain::PublicSuffix
+	if($domain =~ m/\.(com|net|org|edu|gov)$/){
+		#subdomain? 
+		my $count = ($domain =~ tr/\.//);
+		if($count > 1){ 
+			my @p = split(/\./, $domain);
+			$domain = $p[-2] . '.' . $p[-1];
+		}
+	}
+	
+	
+	my $params = {
+	    Action                   => 'GetIdentityVerificationAttributes', 
+	    'Identities.member.1'    => $email, 
+        'Identities.member.2'    => $domain, 
+	};
+	my ($response_code, $response_content) = $self->call_ses($params, {});
+	if ( $self->trace ) {
+		print $response_code . "\n"; 
+        print $response_content . "\n"; 
+	}
+
+	if($response_code eq '200') { 
+	    
+	    #return ($response_code, $response_content); 
+
+        my $r; 
+        my $parser = XML::LibXML->new();
+        my $dom = $parser->parse_string($response_content);
+        my $xpath = XML::LibXML::XPathContext->new($dom);
+        $xpath->registerNs('ns', $aws_email_ns);
+        my @nodes = $xpath->findnodes(
+            '/ns:GetIdentityVerificationAttributesResponse' . 
+            '/ns:GetIdentityVerificationAttributesResult' .
+            '/ns:VerificationAttributes' . 
+            '/ns:entry' . 
+            '/ns:value' . 
+            '/ns:VerificationStatus'
+            );
+        my @a = (); 
+       # print scalar(@nodes) . 'nodes!' ; 
+        foreach my $node (@nodes) {
+            my $text = $node->textContent();
+        	push(@a, $text);
+        }
+        sleep(1); 
+        if($a[0] eq 'Success' || $a[1] eq 'Success'){ 
+            $r = 'Success'; 
+        }
+
+		return ($response_code, $r);
+	}
+	else { 
+		return ($response_code, $response_content);
+	}
+}
 sub verify_sender { 
 	my $self   = shift; 
 	my ($args) = @_; 
 	
 	if(!exists($args->{-email})) { 
-		croak "You MUST pass the, '-email' parameter!"; 
+		croak "You MUST pass the, '-email' paramater!"; 
 	}
 	
 	my $params = {
@@ -182,7 +305,7 @@ sub get_stats {
 	my $params = {
         Action => 'GetSendQuota', 
 	};
-	
+
 	my ($response_code, $response_content) = $self->call_ses($params, {});
 		if ( $self->trace ) {
 			print $response_code . "\n"; 
@@ -356,8 +479,20 @@ sub send_msg {
         'Action'          => 'SendRawEmail'
     };
 
-    return $self->call_ses( $params, {} );
+	my ($response_code, $response_content) = $self->call_ses($params, {});
+		if ( $self->trace ) {
+			print $response_code . "\n"; 
+			print $response_content . "\n"; 
+		}	
 
+	return ($response_code, $response_content);
+
+#	if($response_code eq '200') { 
+#		return ($response_code, $self->get_send_response($response_content));
+#	}
+#	else { 
+#		return ($response_code, $response_content);
+#	}
 }
 
 # Read the credentials from $AWS_CREDENTIALS_FILE file.
@@ -378,11 +513,16 @@ sub read_credentials {
         $line =~ /^\s*(.*?)=(.*?)\s*$/
           or die "Cannot parse credentials entry <$line> in <$file>.";
         my ( $key, $value ) = ( $1, $2 );
-        switch ($key) {
-            case 'AWSAccessKeyId' { $self->AWSAccessKeyId($value) }
-            case 'AWSSecretKey'   { $self->AWSSecretKey($value) }
-            else { die "Unrecognized credential <$key> in <$file>."; }
+        if ($key eq 'AWSAccessKeyId') {
+            $self->AWSAccessKeyId($value);
         }
+        elsif($key eq 'AWSSecretKey') { 
+            $self->AWSSecretKey($value) 
+        }
+        else { 
+            die "Unrecognized credential <$key> in <$file>."; 
+        }
+
     }
     close(FILE);
 }
@@ -427,7 +567,7 @@ sub get_signature_v2 {
     $params{'SignatureMethod'}  = 'HmacSHA256';
     $params{'SignatureVersion'} = '2';
 
-    my $endpoint_name = $endpoint;
+    my $endpoint_name = $self->AWS_endpoint;
     $endpoint_name =~ s!^https?://(.*?)/?$!$1!;
 
     my $data = '';
@@ -520,14 +660,14 @@ sub sign_http {
 
     my $request = shift;
 
-    my $endpoint_name = $endpoint;
+    my $endpoint_name = $self->AWS_endpoint;
     $endpoint_name =~ s!^https?://(.*?)/?$!$1!;
 
     $request->date(time);
     $request->header( 'Host', $endpoint_name );
 
     my $signature;
-    my $use_https = $endpoint =~ m!^https://!;
+    my $use_https = $self->AWS_endpoint =~ m!^https://!;
     if ($use_https) {
         $signature = $self->sign_https_request($request);
     }
@@ -567,21 +707,29 @@ sub call_ses {
 
     %opts   = %$opts;
     %params = %$params;
-
-    $endpoint = $opts{'e'} if defined( $opts{'e'} );
-    my $endpoint_name = $endpoint;
+    
+    if(defined( $opts{'e'} )) { 
+         $self->AWS_endpoint($opts{'e'});
+    }
+        
+    my $endpoint_name = $self->AWS_endpoint;
     $endpoint_name =~ s!^https?://(.*?)(:\d+)?/?$!$1!;
+	
 	if(! defined($self->AWSAccessKeyId ) || ! defined($self->AWSSecretKey)) { 
 		$self->read_credentials;
 	}
 	
     $self->prepare_aws_params;
 
-    switch ($signature_version) {
-        case 'V1'   { $self->sign_v1; }
-        case 'V2'   { $self->sign_v2; }
-        case 'HTTP' { }
-        else { die "Unrecognized signature version <$signature_version>."; }
+    if($signature_version eq 'V1') {
+         $self->sign_v1; 
+    }elsif($signature_version eq 'V2') { 
+        $self->sign_v2; 
+    }elsif($signature_version eq 'HTTP') { 
+        # ... 
+    }
+    else {
+        die "Unrecognized signature version <$signature_version>.";
     }
 
     my $payload = $self->build_payload;
@@ -595,7 +743,7 @@ sub call_ses {
         $browser = $self->browser;
     }
 
-    my $request = new HTTP::Request 'POST', $endpoint;
+    my $request = new HTTP::Request 'POST', $self->AWS_endpoint;
     $request->header( "If-SSL-Cert-Subject" => "/CN=$endpoint_name" );
     $request->content($payload);
     $request->content_type('application/x-www-form-urlencoded');
@@ -605,7 +753,6 @@ sub call_ses {
     my $response = $browser->request($request);
 
     if ( $self->trace ) {
-
         #	carp $response->content;
     }
 
@@ -624,7 +771,29 @@ sub call_ses {
 
     #my $t1 = gettimeofday();
     #carp "mailing time: " . ($t1 - $t0);
-    return ( $response->code, $response->content );
+	
+	return ($response->code, $response->content);
+}
+
+# Gets response sent by amazon
+sub get_send_response {
+	my $self = shift;
+    my $response_content = shift;
+
+    my $parser = XML::LibXML->new();
+    my $dom = $parser->parse_string($response_content);
+    my $xpath = XML::LibXML::XPathContext->new($dom);
+    $xpath->registerNs('ns', $aws_email_ns);
+	
+    my $messageId = $xpath->find('/ns:SendRawEmailResponse' .
+	                             '/ns:SendRawEmailResult' .
+								 '/ns:MessageId');
+
+    my $requestId = $xpath->find('/ns:SendRawEmailResponse' .
+	                             '/ns:ResponseMetadata' .
+	                             '/ns:RequestId');
+
+	return $messageId . "\n" . $requestId;
 }
 
 # Prints tha data returned by the service call.
@@ -654,8 +823,9 @@ sub get_response {
 
 
 
-sub DESTORY {
-    my $self = shift;
-}
+#sub DESTORY {}
+	
 
 1;
+
+__END__
\ No newline at end of file
