@@ -1,17 +1,11 @@
 #!/usr/bin/perl
 use strict; 
 
-
- use Data::Dumper; 
-
-
-
+use Data::Dumper; 
 use lib qw(./t ./ ./DADA/perllib ../ ../DADA/perllib ../../ ../../DADA/perllib 
-	
+
 	/Users/justin/Documents/DadaMail/build/bundle/perllib
-	
-	
-	); 
+); 
 BEGIN{$ENV{NO_DADA_MAIL_CONFIG_IMPORT} = 1}
 use dada_test_config; 
 
@@ -34,7 +28,7 @@ use DADA::MailingList::Subscribers;
 my $lh = DADA::MailingList::Subscribers->new({-list => $list}); 
 my $ls = DADA::MailingList::Settings->new({-list => $list}); 
 
-$ls->save(
+$ls->save({ -settings =>
     {
         use_alt_url_sub_confirm_failed  => 1,
         alt_url_sub_confirm_failed_w_qs => 1,
@@ -51,7 +45,7 @@ $ls->save(
         alt_url_sub_success_w_qs => 1,
         alt_url_sub_success => 'http://example.com/alt_url_sub_success.html',
     }
-);
+});
 
 
 
@@ -164,16 +158,17 @@ ok(! defined($r->{list}));
 
 # diag Dumper($r); 
 undef $r;
-my $r = $das->subscribe(
+my ($h, $b) = $das->subscribe(
     {
         -cgi_obj     => $q,
         -html_output => 1,
     	-return_json => 0, 
 	}
 );
-my $redirect = "Status: 302 Found\r\nLocation: http://www.changetoyoursite.com/cgi-bin/dada/mail.cgi?error_invalid_list=1\r\n\r\n";
-ok($r eq $redirect); 
-undef $r;
+
+ok($h->{-redirect_uri} eq 'http://www.changetoyoursite.com/cgi-bin/dada/mail.cgi?error_invalid_list=1'); 
+undef $h;
+undef $b;
 
 
 
@@ -194,18 +189,18 @@ ok($r->{redirect}->{using} == 1);
 ok($r->{redirect}->{using_with_query} == 1);
 #diag Dumper($r); 
 undef $r; 
-my $r = $das->subscribe(
+my ($h, $b) = $das->subscribe(
     {
         -cgi_obj     => $q,
         -html_output => 1,
     	-return_json => 0, 
 	}
 );
-my $redirect = "Status: 302 Found\r\nLocation: http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=&status=0&rm=sub_confirm&errors[]=invalid_email\r\n\r\n";
-ok($r eq $redirect); 
+my $redirect = 'http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=&status=0&rm=sub_confirm&errors[]=invalid_email';
+ok($h->{-redirect_uri} eq $redirect); 
 $q->delete_all; 
-undef $r;
-
+undef $h;
+undef $b; 
 
 
 
@@ -233,16 +228,18 @@ $lh->remove_subscriber(
     }
 );
 $ct->remove_all_tokens; 
-my $r = $das->subscribe(
+my ($h, $b) = $das->subscribe(
     {
         -cgi_obj     => $q,
         -html_output => 1,
     	-return_json => 0, 
 	}
 );
-my $redirect = "Status: 302 Found\r\nLocation: http://example.com/alt_url_sub_confirm_success.html?list=dadatest&email=user%40example.com&status=1&rm=sub_confirm\r\n\r\n";
-ok($r eq $redirect); 
-undef $r; 
+my $redirect = "http://example.com/alt_url_sub_confirm_success.html?list=dadatest&email=user%40example.com&status=1&rm=sub_confirm";
+ok($h->{-redirect_uri} eq $redirect); 
+undef $h; 
+undef $b; 
+
 $q->delete_all; 
 # Don't remove the email from sub_confirm_list, as we'll use it for the next test: 
 
@@ -258,10 +255,11 @@ my $r = $das->subscribe(
     	-return_json => 0, 
 	}
 );
+diag(Dumper($r)); 
 ok($r->{email} eq 'user@example.com');
 ok($r->{errors}->{already_sent_sub_confirmation} == 1);
 ok($r->{redirect}->{query} eq 'list=dadatest&email=user%40example.com&status=0&rm=sub_confirm&errors[]=already_sent_sub_confirmation');
-ok($r->{redirect}->{url}   eq 'http://localhost?f=show_error&email=user%40example.com&list=dadatest&error=already_sent_sub_confirmation');
+ok($r->{redirect}->{url}   eq 'http://www.changetoyoursite.com/cgi-bin/dada/mail.cgi?flavor=show_error&email=user%40example.com&list=dadatest&error=already_sent_sub_confirmation');
 ok($r->{redirect}->{using} == 1); 
 ok($r->{redirect}->{using_with_query} == 1);
 ok($r->{status} == 0);
@@ -271,16 +269,18 @@ ok($r->{list} eq $list);
 
 
 undef $r; 
-my $r = $das->subscribe(
+my ($h, $b) = $das->subscribe(
     {
         -cgi_obj     => $q,
         -html_output => 1,
     	-return_json => 0, 
 	}
 );
-my $redirect = "Status: 302 Found\r\nLocation: http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=user%40example.com&status=0&rm=sub_confirm&errors[]=already_sent_sub_confirmation\r\n\r\n";
-ok($r eq $redirect); 
-undef $r; 
+my $redirect = 'http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=user%40example.com&status=0&rm=sub_confirm&errors[]=already_sent_sub_confirmation';
+ok($h->{-redirect_uri} eq $redirect); 
+undef $h;
+undef $b; 
+ 
 $q->delete_all;
 $lh->remove_subscriber(
     {
@@ -293,8 +293,8 @@ $ct->remove_all_tokens;
 
 
 # Black Listed, but OK for subscriber to re-subscribe: 
-$ls->save({black_list => 1}); 
-$ls->save({allow_blacklisted_to_subscribe => 1}); 
+$ls->save({ -settings => {black_list => 1}}); 
+$ls->save({ -settings => {allow_blacklisted_to_subscribe => 1}}); 
 $lh->add_subscriber( { -email => $email, -type  => 'black_list', } );
 $q->param('list',  $list); 
 $q->param('email', $email); 
@@ -328,16 +328,18 @@ $lh->remove_subscriber(
         -type  => 'black_list',
     }
 );
-my $r = $das->subscribe(
+my ($h, $b) =  $das->subscribe(
     {
         -cgi_obj     => $q,
         -html_output => 1,
     	-return_json => 0, 
 	}
 );
-my $redirect = "Status: 302 Found\r\nLocation: http://example.com/alt_url_sub_confirm_success.html?list=dadatest&email=user%40example.com&status=1&rm=sub_confirm\r\n\r\n";
-ok($r eq $redirect); 
-undef $r;
+my $redirect = 'http://example.com/alt_url_sub_confirm_success.html?list=dadatest&email=user%40example.com&status=1&rm=sub_confirm';
+ok($h->{-redirect_uri} eq $redirect); 
+undef $h;
+undef $b;
+
 $ct->remove_all_tokens; 
 $lh->remove_subscriber(
     {
@@ -356,8 +358,8 @@ $q->delete_all;
 
 
 # black_listed
-$ls->save({black_list => 1}); 
-$ls->save({allow_blacklisted_to_subscribe => 0}); 
+$ls->save({ -settings => {black_list => 1}}); 
+$ls->save({ -settings => {allow_blacklisted_to_subscribe => 0}}); 
 $lh->add_subscriber( { -email => $email, -type  => 'black_list', } );
 
 $q->param('list',  $list); 
@@ -383,16 +385,19 @@ ok($r->{list} eq $list);
 undef $r; 
 $ct->remove_all_tokens;
 
-my $r = $das->subscribe(
+my ($h, $b) =  $das->subscribe(
     {
         -cgi_obj     => $q,
         -html_output => 1,
     	-return_json => 0, 
 	}
 );
-my $redirect = "Status: 302 Found\r\nLocation: http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=user%40example.com&status=0&rm=sub_confirm&errors[]=black_listed\r\n\r\n";
-ok($r eq $redirect); 
-undef $r;
+my $redirect = 'http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=user%40example.com&status=0&rm=sub_confirm&errors[]=black_listed';
+ok($h->{-redirect_uri} eq $redirect); 
+undef $h;
+undef $b;
+
+
 $lh->remove_subscriber(
     {
         -email => $email,
@@ -407,7 +412,7 @@ $q->delete_all;
 
 
 # closed_list 
-$ls->save({closed_list => 1}); 
+$ls->save({ -settings => {closed_list => 1}}); 
 $q->param('list',  $list); 
 $q->param('email', $email); 
 my $r = $das->subscribe(
@@ -427,22 +432,24 @@ ok($r->{redirect}->{using} == 1);
 ok($r->{redirect}->{using_with_query} == 1);
 ok(exists($r->{error_descriptions}->{closed_list})); 
 undef $r; 
-my $r = $das->subscribe(
+my ($h, $b) =  $das->subscribe(
     {
         -cgi_obj     => $q,
         -html_output => 1,
     	-return_json => 0, 
 	}
 );
-my $redirect = "Status: 302 Found\r\nLocation: http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=user%40example.com&status=0&rm=sub_confirm&errors[]=closed_list\r\n\r\n";
-ok($r eq $redirect); 
+my $redirect = 'http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=user%40example.com&status=0&rm=sub_confirm&errors[]=closed_list';
+ok($h->{-redirect_uri} eq $redirect); 
+undef $h;
+undef $b;
+
 $q->delete_all;
-$ls->save({closed_list => 0});  
-undef $r; 
+$ls->save({ -settings => {closed_list => 0}});  
 
 
 # invite_only_list 
-$ls->save({invite_only_list => 1}); 
+$ls->save({ -settings => {invite_only_list => 1}}); 
 $q->param('list',  $list); 
 $q->param('email', $email); 
 my $r = $das->subscribe(
@@ -465,25 +472,26 @@ ok(exists($r->{error_descriptions}->{invite_only_list}));
 
 #diag Dumper($r); 
 undef $r; 
-my $r = $das->subscribe(
+my ($h, $b) = $das->subscribe(
     {
         -cgi_obj     => $q,
         -html_output => 1,
     	-return_json => 0, 
 	}
 );
-my $redirect = "Status: 302 Found\r\nLocation: http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=user%40example.com&status=0&rm=sub_confirm&errors[]=invite_only_list\r\n\r\n";
-ok($r eq $redirect); 
-undef $r; 
+my $redirect = 'http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=user%40example.com&status=0&rm=sub_confirm&errors[]=invite_only_list';
+ok($h->{-redirect_uri} eq $redirect); 
+undef $h;
+undef $b;
 $q->delete_all; 
-$ls->save({invite_only_list => 0}); 
+$ls->save({ -settings => {invite_only_list => 0}}); 
 
 
 
 
 # Over Subscription Quota 
-$ls->save({use_subscription_quota => 1}); 
-$ls->save({subscription_quota     => 0}); 
+$ls->save({ -settings => {use_subscription_quota => 1}}); 
+$ls->save({ -settings => {subscription_quota     => 0}}); 
 $q->param('list',  $list); 
 $q->param('email', $email); 
 my $r = $das->subscribe(
@@ -504,18 +512,21 @@ ok($r->{redirect}->{using_with_query} == 1);
 ok(exists($r->{error_descriptions}->{over_subscription_quota})); 
 #diag Dumper($r); 
 undef $r; 
-my $r = $das->subscribe(
+my ($h, $b) = $das->subscribe(
     {
         -cgi_obj     => $q,
         -html_output => 1,
     	-return_json => 0, 
 	}
 );
-my $redirect = "Status: 302 Found\r\nLocation: http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=user%40example.com&status=0&rm=sub_confirm&errors[]=over_subscription_quota\r\n\r\n";
-ok($r eq $redirect); 
-undef $r; 
+my $redirect = 'http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=user%40example.com&status=0&rm=sub_confirm&errors[]=over_subscription_quota';
+
+ok($h->{-redirect_uri} eq $redirect); 
+undef $h;
+undef $b;
+
 $q->delete_all; 
-$ls->save({use_subscription_quota => 0});
+$ls->save({ -settings => {use_subscription_quota => 0}});
 
 
 
@@ -544,24 +555,119 @@ ok($r->{redirect}->{using_with_query} == 1);
 ok(exists($r->{error_descriptions}->{over_subscription_quota})); 
 #diag Dumper($r);
 undef $r; 
-my $r = $das->subscribe(
+my ($h, $b) =  $das->subscribe(
     {
         -cgi_obj     => $q,
         -html_output => 1,
     	-return_json => 0, 
 	}
 );
-my $redirect = "Status: 302 Found\r\nLocation: http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=user%40example.com&status=0&rm=sub_confirm&errors[]=over_subscription_quota\r\n\r\n";
-ok($r eq $redirect);  
-undef $r; 
+my $redirect = 'http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=user%40example.com&status=0&rm=sub_confirm&errors[]=over_subscription_quota';
+
+ok($h->{-redirect_uri} eq $redirect); 
+undef $h;
+undef $b;
+
 $q->delete_all;
 $DADA::Config::SUBSCRIPTION_QUOTA = undef; 
 
 
 
+
+# Closed Loop Opt-in DISABLED
+$ls->save({ -settings => 
+    {
+        enable_closed_loop_opt_in         => 0,
+    }
+});
+
+$q->param('list',  $list); 
+$q->param('email', $email); 
+my $r = $das->subscribe(
+    {
+        -cgi_obj     => $q,
+        -html_output => 0,
+    	-return_json => 0, 
+	}
+);
+#diag(Dumper($r)); 
+
+ok($r->{email} eq $email);
+ok($r->{list} eq $list);
+ok($r->{status} == 1);
+
+ok($r->{redirect}->{url}   eq 'http://example.com/alt_url_sub_success.html');
+ok($r->{redirect}->{query} eq 'list=dadatest&rm=sub&status=1&email=user%40example.com');
+ok($r->{redirect}->{using} == 1); 
+ok($r->{redirect}->{using_with_query} == 1);
+ok(exists($r->{success_message})); 
+
+$lh->remove_subscriber( { -email => $email, -type  => 'list', } );
+undef $r; 
+$q->delete_all;
+$ls->save({ -settings => 
+    {
+        enable_closed_loop_opt_in         => 1,
+    }
+});
+
+
+
+
+# Closed Loop Opt-in DISABLED
+# Subscription Requests ENABLED
+$ls->save({ -settings => 
+    {
+        enable_closed_loop_opt_in         => 0,
+        enable_subscription_approval_step => 1, 
+    }
+});
+
+$q->param('list',  $list); 
+$q->param('email', $email); 
+my $r = $das->subscribe(
+    {
+        -cgi_obj     => $q,
+        -html_output => 0,
+    	-return_json => 0, 
+	}
+);
+ok($r->{email} eq $email);
+ok($r->{list} eq $list);
+ok($r->{status} == 1);
+ok($r->{redirect}->{url}   eq '');
+ok($r->{redirect}->{query} eq 'list=dadatest&rm=sub&subscription_requires_approval=1&status=1&email=user%40example.com');
+ok($r->{redirect}->{using} == 0); 
+ok($r->{redirect}->{using_with_query} == 0);
+ok(exists($r->{success_message})); 
+#diag Dumper($r);
+
+$lh->remove_subscriber( { -email => $email, -type  => 'sub_confirm_list', } );
+undef $r; 
+$q->delete_all;
+$ls->save({ -settings => 
+    {
+        enable_closed_loop_opt_in         => 1,
+        enable_subscription_approval_step => 0, 
+    }
+});
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 # Subscribed - w/o email_your_subscribed_msg enabled
 # This is tricky, since by default, DM lies about subscriptions list this: 
-$ls->save({ email_your_subscribed_msg => 1 }); 
+$ls->save({ -settings => { email_your_subscribed_msg => 1 }}); 
 $lh->add_subscriber( { -email => $email, -type  => 'list', } );
 $q->param('list',  $list); 
 $q->param('email', $email); 
@@ -586,16 +692,20 @@ ok(exists($r->{success_message}));
 $lh->remove_subscriber( { -email => $email, -type  => 'list', } );
 $lh->remove_subscriber( { -email => $email, -type  => 'sub_confirm_list', } );
 undef $r; 
-my $r = $das->subscribe(
+my ($h, $b) =  $das->subscribe(
     {
         -cgi_obj     => $q,
         -html_output => 1,
     	-return_json => 0, 
 	}
 );
-my $redirect = "Status: 302 Found\r\nLocation: http://example.com/alt_url_sub_confirm_success.html?list=dadatest&email=user%40example.com&status=1&rm=sub_confirm\r\n\r\n";
-ok($r eq $redirect); 
-undef $r; 
+my $redirect = 'http://example.com/alt_url_sub_confirm_success.html?list=dadatest&email=user%40example.com&status=1&rm=sub_confirm';
+
+ok($h->{-redirect_uri} eq $redirect); 
+undef $h;
+undef $b;
+
+
 $q->delete_all;
 $lh->remove_subscriber( { -email => $email, -type  => 'list', } );
 $lh->remove_subscriber( { -email => $email, -type  => 'sub_confirm_list', } );
@@ -605,7 +715,7 @@ $lh->remove_subscriber( { -email => $email, -type  => 'sub_confirm_list', } );
 
 # Subscribed - with email_your_subscribed_msg DISABLED
 $lh->add_subscriber( { -email => $email, -type  => 'list', } );
-$ls->save({ email_your_subscribed_msg => 0 }); 
+$ls->save({ -settings => { email_your_subscribed_msg => 0 }}); 
 $q->param('list',  $list); 
 $q->param('email', $email); 
 my $r = $das->subscribe(
@@ -627,40 +737,34 @@ ok(exists($r->{error_descriptions}->{subscribed}));
 
 
 undef $r; 
-my $r = $das->subscribe(
+my ($h, $b) =  $das->subscribe(
     {
         -cgi_obj     => $q,
         -html_output => 1,
     	-return_json => 0, 
 	}
 );
-my $redirect = "Status: 302 Found\r\nLocation: http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=user%40example.com&status=0&rm=sub_confirm&errors[]=subscribed\r\n\r\n";
-ok($r eq $redirect); 
-undef $r; 
+my $redirect = 'http://example.com/alt_url_sub_confirm_failed.html?list=dadatest&email=user%40example.com&status=0&rm=sub_confirm&errors[]=subscribed';
+
+ok($h->{-redirect_uri} eq $redirect); 
+undef $h;
+undef $b;
+
+
 $lh->remove_subscriber( { -email => $email, -type  => 'list', } );
 $q->delete_all;
 
-$ls->save(
+$ls->save({ -settings => 
 	{ 
 		enable_closed_loop_opt_in         => 0, 
 		captcha_sub                       => 1, 
 		enable_subscription_approval_step => 0, 
 		
 	}
-); 
+}); 
 $q->param('list',  $list); 
 $q->param('email', $email); 
-#my $r = $das->subscribe(
-#    {
-#        -cgi_obj     => $q,
-#        -html_output => 1,
-#    	-return_json => 0, 
-#	}
-#);
-#diag $r; 
-#
-#
-#undef $r;
+
 my $r = $das->subscribe(
     {
         -cgi_obj     => $q,
@@ -671,25 +775,120 @@ my $r = $das->subscribe(
 ok($r->{email} eq $email);
 ok($r->{list} eq $list);
 ok($r->{status} == 1);
-ok($r->{redirect}->{url}   eq 'http://localhost?f=subscribe&email=user%40example.com&list=dadatest');
+diag($r->{redirect}->{url}); 
+ok($r->{redirect}->{url}   eq 'http://www.changetoyoursite.com/cgi-bin/dada/mail.cgi?flavor=subscribe&email=user%40example.com&list=dadatest');
 ok($r->{redirect_required} eq 'subscription_requires_captcha'); 
 ok($r->{success_message} eq 'use redirect'); 
 undef $r; 
-
-#my $r = $das->subscribe(
-#    {
-#        -cgi_obj     => $q,
-#        -html_output => 0,
-#    	-return_json => 1, 
-#	}
-#);
-#
-#diag $r; 
-
-
 $q->delete_all;
 
 
+
+SKIP: {
+    skip "Profile Fields is not supported with this current backend." 
+    if $lh->can_have_subscriber_fields == 0; 
+
+    # Profile Fields
+
+    require DADA::ProfileFieldsManager;
+    my $pfm = DADA::ProfileFieldsManager->new;
+    $pfm->add_field(
+        {
+            -field          => 'first_name',
+            -required       => 1,
+        }
+    );
+    $pfm->add_field(
+        {
+            -field          => 'last_name',
+            -required       => 0,
+        }
+    );
+
+    $q->param('list',  $list); 
+    $q->param('email', $email); 
+    my $r = $das->subscribe(
+        {
+            -cgi_obj     => $q,
+            -html_output => 0,
+        	-return_json => 0, 
+    	}
+    );
+    #diag Dumper($r);
+    ok($r->{email} eq $email);
+    ok($r->{list} eq $list);
+    ok($r->{status} == 0);
+    #ok($r->{redirect}->{url}   eq '');
+    #ok($r->{redirect}->{query} eq 'list=dadatest&rm=sub&subscription_requires_approval=1&status=1&email=user%40example.com');
+    #ok($r->{redirect}->{using} == 0); 
+    #ok($r->{redirect}->{using_with_query} == 0);
+    ok(exists($r->{error_descriptions}->{invalid_profile_fields})); 
+    ok($r->{profile_errors}->{first_name}->{required} == 1); 
+    undef $r;
+    $q->delete_all;
+    $lh->remove_subscriber(
+        {
+            -email => $email,
+            -type  => 'sub_confirm_list',
+        }
+    );
+    $ct->remove_all_tokens; 
+
+
+
+    # This one, we just make sure it works, with profile fields, and those FIELDS ARE SAVED!!! 
+    undef $lh; 
+    my $lh = DADA::MailingList::Subscribers->new({-list => $list}); 
+
+    $ls->save({ -settings => 
+    	{ 
+    		enable_closed_loop_opt_in         => 1, 
+    		captcha_sub                       => 0, 
+    		enable_subscription_approval_step => 0, 		
+    	}
+    }); 
+
+    $q->param('list',  $list); 
+    $q->param('email', $email); 
+    $q->param('first_name', 'First Name!'); 
+    $q->param('last_name', 'Last Name!'); 
+
+    my $r = $das->subscribe(
+        {
+            -cgi_obj     => $q,
+            -html_output => 0,
+        	-return_json => 0, 
+    	}
+    );
+    #diag Dumper($r);
+    ok($r->{email} eq $email);
+    ok($r->{list} eq $list);
+    ok($r->{status} == 1);
+    #ok($r->{redirect}->{url}   eq '');
+    #ok($r->{redirect}->{query} eq 'list=dadatest&rm=sub&subscription_requires_approval=1&status=1&email=user%40example.com');
+    #ok($r->{redirect}->{using} == 0); 
+    #ok($r->{redirect}->{using_with_query} == 0);
+
+    #sleep(30); 
+
+    my $sub_info = $lh->get_subscriber(
+    		{
+    			-email => '*' . $email, 
+    			-type  => 'sub_confirm_list', 
+    		}
+    	);
+    ok($sub_info->{'first_name'} eq 'First Name!');
+    ok($sub_info->{'last_name'}  eq 'Last Name!');
+
+    undef $r; 
+    $q->delete_all;
+
+};
+
+
+
+
+
 dada_test_config::remove_test_list;
 dada_test_config::remove_test_list({-name => 'test2'});
 dada_test_config::wipe_out;
\ No newline at end of file
