@@ -5,6 +5,8 @@ my $large_num = 1000;
 use lib
   qw(./t ./ ./DADA/perllib ../ ../DADA/perllib ../../ ../../DADA/perllib );
 BEGIN { $ENV{NO_DADA_MAIL_CONFIG_IMPORT} = 1 }
+use DADA::Config; 
+
 use dada_test_config;
 
 #diag('$DADA::Config::SUBSCRIBER_DB_TYPE ' . $DADA::Config::SUBSCRIBER_DB_TYPE);
@@ -20,140 +22,22 @@ my $list = dada_test_config::create_test_list(
     { -remove_existing_list => 1, -remove_subscriber_fields => 1 } );
 
 use DADA::MailingList::Subscribers;
+my $lh = DADA::MailingList::Subscribers->new( { -list => $list } );
 
-# Filter stuff
+use DADA::MailingList::Settings;
+my $ls = DADA::MailingList::Settings->new( { -list => $list } );
 
-my @email_list = qw(
-
-  user@example.com
-  user
-  example.com
-  @example.com
-
-);
 
 chmod( 0777, $DADA::Config::TMP . '/mail.txt' );
 
-my $lh = DADA::MailingList::Subscribers->new( { -list => $list } );
-
 ok(
     $lh->{ls}->isa('DADA::MailingList::Settings'),
     "looks like there's a Settings obj in ->{ls}, good!"
 );
 
-my ( $subscribed, $not_subscribed, $black_listed, $not_white_listed, $invalid )
-  = $lh->filter_subscribers( { -emails => [@email_list], } );
 
-ok( eq_array( $subscribed, [] ) == 1, "Subscribed" );
-ok( eq_array( $not_subscribed, ['user@example.com'] ) == 1,
-    "Not Subscribed (user\@example.com)" );
-ok( eq_array( $black_listed,     [] ) == 1, "Black Listed" );
-ok( eq_array( $not_white_listed, [] ) == 1, "Not White Listed" );
-ok(
-    eq_array( [ sort(@$invalid) ],
-        [ sort( 'user', 'example.com', '@example.com' ) ] ) == 1,
-    "Invalid"
-);
-
-undef($subscribed);
-undef($not_subscribed);
-undef($black_listed);
-undef($not_white_listed);
-undef($invalid);
-
-#diag '$lh->num_subscribers ' . $lh->num_subscribers;
-
-my $count = $lh->add_to_email_list(
-    -Email_Ref => [
-        qw(
-          user@example.com
-          )
-    ],
-    -Type => 'list',
-);
-
-ok( $count == 1, "added one address" );
-undef($count);
-
-my ( $subscribed, $not_subscribed, $black_listed, $not_white_listed, $invalid )
-  = $lh->filter_subscribers( { -emails => [@email_list], } );
-
-ok( eq_array( $subscribed, ['user@example.com'] ) == 1, "Subscribed" );
-ok( eq_array( $not_subscribed,   [] ) == 1, "Not Subscribed ()" );
-ok( eq_array( $black_listed,     [] ) == 1, "Black Listed" );
-ok( eq_array( $not_white_listed, [] ) == 1, "Not White Listed" );
-ok(
-    eq_array( [ sort(@$invalid) ],
-        [ sort( 'user', 'example.com', '@example.com' ) ] ) == 1,
-    "Invalid"
-);
-
-undef($subscribed);
-undef($not_subscribed);
-undef($black_listed);
-undef($not_white_listed);
-undef($invalid);
-
-#diag '$lh->num_subscribers ' . $lh->num_subscribers;
-
-my $r_count = $lh->remove_from_list( -Email_List => ['user@example.com'] );
-ok( $r_count == 1, "removed one address (user\@example.com)" );
-undef($r_count);
-
-use DADA::MailingList::Settings;
-my $ls = DADA::MailingList::Settings->new( { -list => $list } );
-$ls->save( { black_list => 1 } );
-my $li = $ls->get();
-ok( $li->{black_list} == 1, "black list enabled." );
-
-# 'user@', '@example.com', '@' , 'u'
-
-for my $blacklist_this ( 'user@', '@example.com', 'user@example.com' ) {
-
-    #diag '$blacklist_this - ' . $blacklist_this;
-
-    my $count = $lh->add_to_email_list(
-        -Email_Ref => [ $blacklist_this ],
-        -Type      => 'black_list',
-    );
-
-    ok( $count == 1, "added one address" );
-    undef($count);
-
-    my (
-        $subscribed,       $not_subscribed, $black_listed,
-        $not_white_listed, $invalid
-    ) = $lh->filter_subscribers( { -emails => [@email_list], } );
-
-    ok( eq_array( $subscribed,     [] ) == 1, "Subscribed" );
-    ok( eq_array( $not_subscribed, [] ) == 1, "Not Subscribed ()" );
-    ok( eq_array( $black_listed, ['user@example.com'] ) == 1,
-        "Black Listed (user\@example.com)" );
-    ok( eq_array( $not_white_listed, [] ) == 1, "Not White Listed" );
-    ok(
-        eq_array( [ sort(@$invalid) ],
-            [ sort( 'user', 'example.com', '@example.com' ) ] ) == 1,
-        "Invalid"
-    );
-
-    undef($subscribed);
-    undef($not_subscribed);
-    undef($black_listed);
-    undef($not_white_listed);
-    undef($invalid);
-
-    my $r_count = $lh->remove_from_list(
-        -Email_List => [$blacklist_this],
-        -Type       => 'black_list'
-    );
-    ok( $r_count == 1, "removed one address from blacklist ($blacklist_this)" );
-    undef($r_count);
-
-}
-
-$ls->save( { black_list => 0 } );
-$li = $ls->get();
-ok( $li->{black_list} == 0, "black list disabled." );
+$ls->save({ -settings =>  { black_list => 0 } });
+ok( $ls->param('black_list') == 0, "black list disabled." );
 
 # Oh geez. This should have a million tests.
 my @good_addresses =
@@ -264,19 +148,19 @@ ok( $@,
 );
 
 ok(
-    $lh->remove_from_list(
-        -Email_List => ['mytest@example.com'],
-        -Type       => 'list'
-    )
+    $lh->remove_subscriber({ 
+        -email => 'mytest@example.com',
+        -type       => 'list'
+    })
 );
 ok(
-    $lh->remove_from_list(
-        -Email_List => ['mytest@example.com'],
-        -Type       => 'black_list'
-    )
+    $lh->remove_subscriber({
+        -email => 'mytest@example.com',
+        -type       => 'black_list',
+    })
 );
 
-diag "Justin! there are, " . $lh->num_subscribers . "on this list."; 
+#diag "Justin! there are, " . $lh->num_subscribers . "on this list."; 
 
 # So, add_subscriber actually doesn't have a dupe check by default, so it is possible to 
 # add the same address multiple times. Most of the app knows this, and takes pains to make sure dupe check is enabled, 
@@ -460,17 +344,17 @@ ok(
     "Calling copy_subscriber with correct parameters works!"
 );
 ok(
-    $lh->remove_from_list(
-        -Email_List => ['mytest@example.com'],
-        -Type       => 'list'
-    ),
+    $lh->remove_subscriber({
+        -email => 'mytest@example.com',
+        -type  => 'list',
+    }),
     'removed mytest@example.com list'
 );
 ok(
-    $lh->remove_from_list(
-        -Email_List => ['mytest@example.com'],
-        -Type       => 'black_list'
-    ),
+    $lh->remove_subscriber({
+        -email => 'mytest@example.com',
+        -type       => 'black_list'
+    }),
     'removed mytest@example.com black_list'
 );
 
@@ -479,7 +363,7 @@ ok(
 SKIP: {
 
     skip
-"Multiple Subscriber Profile Fields is not supported with this current backend."
+"Multiple Profile Fields is not supported with this current backend."
       if $lh->can_have_subscriber_fields == 0;
 
     ok( eq_array( $lh->subscriber_fields, [] ),
@@ -750,7 +634,7 @@ SKIP: {
 
     ###
 
-    #### copy_subscriber w/Subscriber Profile Fields
+    #### copy_subscriber w/Profile Fields
     # the idea is that the subscriber field information
     # should be copied over correctly as well.
 
@@ -826,17 +710,17 @@ SKIP: {
     undef $one_info;
 
     ok(
-        $lh->remove_from_list(
-            -Email_List => ['one@example.com'],
-            -Type       => 'list'
-        ),
+        $lh->remove_subscriber({
+            -email => 'one@example.com',
+            -type       => 'list'
+        }),
         'removed one@example.com from list'
     );
     ok(
-        $lh->remove_from_list(
-            -Email_List => ['one@example.com'],
-            -Type       => 'black_list'
-        ),
+        $lh->remove_subscriber({
+            -email => 'one@example.com',
+            -Type  => 'black_list'
+        }),
         'removed one@example.com from black_list'
     );
 
@@ -994,10 +878,10 @@ SKIP: {
 
     my $body = q{ 
     
-    email: [subscriber.email], First Name: [subscriber.first_name], Last Name: [subscriber.last_name]
-    
-    [subscriber.first_name][subscriber.first_name][subscriber.first_name][subscriber.first_name][subscriber.first_name][subscriber.first_name]
-    [subscriber.last_name][subscriber.last_name][subscriber.last_name][subscriber.last_name][subscriber.last_name][subscriber.last_name][subscriber.last_name]
+    email: <!-- tmpl_var subscriber.email -->, First Name: <!-- tmpl_var subscriber.first_name -->, Last Name: <!-- tmpl_var subscriber.last_name -->
+
+    <!-- tmpl_var subscriber.first_name --><!-- tmpl_var subscriber.first_name --><!-- tmpl_var subscriber.first_name --><!-- tmpl_var subscriber.first_name --><!-- tmpl_var subscriber.first_name --><!-- tmpl_var subscriber.first_name -->
+    <!-- tmpl_var subscriber.last_name --><!-- tmpl_var subscriber.last_name --><!-- tmpl_var subscriber.last_name --><!-- tmpl_var subscriber.last_name --><!-- tmpl_var subscriber.last_name --><!-- tmpl_var subscriber.last_name --><!-- tmpl_var subscriber.last_name -->
     
     };
 
@@ -1053,10 +937,10 @@ SKIP: {
 
     close $TMP_SUB_LIST or die $!;
 
-    like( $contents, qr/Mike\,Kelley/ );
-    like( $contents, qr/Raymond\,Pettibon/ );
-    like( $contents, qr/Marcel\,Duchamp/ );
-    like( $contents, qr/Man\,Ray/ );
+    like( $contents, qr/Mike\"\,\"Kelley/ );
+    like( $contents, qr/Raymond\"\,\"Pettibon/ );
+    like( $contents, qr/Marcel\"\,\"Duchamp/ );
+    like( $contents, qr/Man\"\,\"Ray/ );
 
     # ok($contents =~ m/John\:\:Doe/);
 
@@ -1069,7 +953,7 @@ SKIP: {
 
   #diag( 'orig: $DADA::Config::MAIL_SETTINGS ' . $DADA::Config::MAIL_SETTINGS );
 
-    $ls->save(
+    $ls->save({ -settings => 
         {
             enable_bulk_batching => 0,
 
@@ -1080,7 +964,7 @@ SKIP: {
             html_encoding      => '8bit',
 
         }
-    );
+    });
 
     unlink $DADA::Config::TMP . '/mail.txt'
       if ( -e $DADA::Config::TMP . '/mail.txt' );
@@ -1092,7 +976,7 @@ SKIP: {
     my $mh = DADA::Mail::Send->new( { -list => $list } );
     $mh->test_send_file( $DADA::Config::TMP . '/mail.txt' );
     $mh->test(1);
-    my $msg_id = $mh->mass_send( %$test_msg_fields, );
+    my $msg_id = $mh->mass_send({ -msg => {%$test_msg_fields }});
 
     my $mailout = DADA::Mail::MailOut->new( { -list => $list } );
     my $associate_worked = $mailout->associate( $msg_id, 'list' );
@@ -1163,6 +1047,9 @@ SKIP: {
       'email: mike.kelley@example.com, First Name: Mike, Last Name: Kelley';
     my $l2 = 'MikeMikeMikeMikeMikeMike';
     my $l3 = 'KelleyKelleyKelleyKelleyKelleyKelleyKelley';
+    
+    # diag '$contents ' . $contents; 
+    
     like( $contents, qr/$l/ );
     like( $contents, qr/$l2/ );
     like( $contents, qr/$l3/ );
@@ -1208,7 +1095,6 @@ SKIP: {
 
 
     # We'll just do this again, for good measure...
-
     unlink $DADA::Config::TMP . '/mail.txt'
       if ( -e $DADA::Config::TMP . '/mail.txt' );
 
@@ -1221,12 +1107,26 @@ SKIP: {
     my $mh = DADA::Mail::Send->new( { -list => $list } );
     $mh->test_send_file( $DADA::Config::TMP . '/mail.txt' );
     $mh->test(1);
-    $mh->partial_sending( { 'first_name' => { 'equal_to' => 'Raymond' } } );
+    $mh->partial_sending( 
+        { 
+            first_name => {
+                -operator => '=',  
+                -value => 'Raymond', 
+            } 
+        } 
+    );
     my $mess_id = '<'
       . DADA::App::Guts::message_id() . '.'
       . DADA::Security::Password::generate_rand_string('1234567890') . '@'
       . 'some-example-with-dashes.com' . '>';
-    my $msg_id = $mh->mass_send( %$test_msg_fields, 'Message-ID' => $mess_id, );
+    my $msg_id = $mh->mass_send(
+		{ 
+			-msg => { 
+				%$test_msg_fields, 
+				'Message-ID' => $mess_id
+			}
+		}
+	);
 
     my $mailout = DADA::Mail::MailOut->new( { -list => $list } );
     my $associate_worked = $mailout->associate( $msg_id, 'list' );
@@ -1274,16 +1174,16 @@ SKIP: {
     my $contents = do { local $/; <$TMP_MAIL_FILE> };
     close $TMP_MAIL_FILE or die $!;
 
-    my $l = quotemeta(
-        'email: mike.kelley@example.com, First Name: Mike, Last Name: Kelley');
+    diag '$contents: ' . $contents;
+
+
+    my $l = quotemeta('email: mike.kelley@example.com, First Name: Mike, Last Name: Kelley');
     my $l2 = quotemeta('MikeMikeMikeMikeMikeMike');
     my $l3 = quotemeta('KelleyKelleyKelleyKelleyKelleyKelleyKelley');
 
-    #diag '$contents: ' . $contents;
-
-    ok( $contents !~ m/$l/ );
-    ok( $contents !~ m/$l2/ );
-    ok( $contents !~ m/$l3/ );
+    ok( $contents !~ m/$l/,  "Found: '$l'");
+    ok( $contents !~ m/$l2/, "Found: '$l2'");
+    ok( $contents !~ m/$l3/, "Found: '$l3'");
     undef $l;
     undef $l2;
     undef $l2;
@@ -1301,13 +1201,12 @@ SKIP: {
     undef $l2;
     undef $l2;
 
-    my $l =
-      quotemeta('email: man.ray@example.com, First Name: Man, Last Name: Ray');
+    my $l =  quotemeta('email: man.ray@example.com, First Name: Man, Last Name: Ray');
     my $l2 = quotemeta('ManManManManManMan');
     my $l3 = quotemeta('RayRayRayRayRayRayRay');
-    ok( $contents !~ m/$l/ );
-    ok( $contents !~ m/$l2/ );
-    ok( $contents !~ m/$l3/, 'DID NOT FIND: RayRayRayRayRayRayRay' );
+    ok( $contents !~ m/$l/,  "Did not find: $l");
+    ok( $contents !~ m/$l2/, "Did not find: $l2");
+    ok( $contents !~ m/$l3/, "Did not find: $l3"); 
     undef $l;
     undef $l2;
     undef $l2;
@@ -1321,9 +1220,7 @@ SKIP: {
       $lh->create_mass_sending_file(
         -ID   => DADA::App::Guts::message_id(),    #argh. That's messy.
         -Type => 'list'
-
-          #-partial_sending => $args->{-partial_sending},
-      );
+     );
 
     ok( ($lh->num_subscribers + 1) == $total_sending_out_num,
         "This file is to send to " . $total_sending_out_num . 'subscribers' );
@@ -1334,7 +1231,7 @@ SKIP: {
 
 
 	# this is to make sure the mass_mailing_send_to_list_owner pref works
-	$ls->save({mass_mailing_send_to_list_owner => 0}); 
+	$ls->save({ -settings => {mass_mailing_send_to_list_owner => 0}}); 
 	# reset the setttings...
     undef $lh;
        my $lh = DADA::MailingList::Subscribers->new( { -list => $list } );	
@@ -1353,7 +1250,7 @@ SKIP: {
     ok( unlink($path_to_list) == 1, 'Unlinking ' . $path_to_list . ' worked.' );
     undef($path_to_list);
     undef($total_sending_out_num);
-	$ls->save({mass_mailing_send_to_list_owner => 1}); 
+	$ls->save({ -settings => {mass_mailing_send_to_list_owner => 1}}); 
 	#/ this is to make sure the mass_mailing_send_to_list_owner pref works
 	undef $lh;
        my $lh = DADA::MailingList::Subscribers->new( { -list => $list } );	
@@ -1366,7 +1263,12 @@ SKIP: {
       $lh->create_mass_sending_file(
         -ID   => DADA::App::Guts::message_id(),    #argh. That's messy.
         -Type => 'list',
-        -partial_sending => { 'first_name' => { 'equal_to' => 'Raymond' } },
+        -partial_sending => { 
+            'first_name' => { 
+                -operator => '=',
+                -value => 'Raymond', 
+            } 
+        },
       );
 
     ok(
@@ -1387,7 +1289,12 @@ SKIP: {
       $lh->create_mass_sending_file(
         -ID   => DADA::App::Guts::message_id(),    #argh. That's messy.
         -Type => 'list',
-        -partial_sending => { 'first_name' => { 'like' => 'a' } },
+        -partial_sending => { 
+            'first_name' => { 
+                -operator => 'LIKE', 
+                -value    => 'a'
+            } 
+        },
       );
 
     ok(
@@ -1404,7 +1311,7 @@ SKIP: {
       $lh->create_mass_sending_file(
         -ID   => DADA::App::Guts::message_id(),    #argh. That's messy.
         -Type => 'list',
-        -partial_sending => { 'last_name' => { 'like' => 'a' } },
+        -partial_sending => { 'last_name' => { -operator => 'LIKE', -value => 'a' } },
       );
 
     #diag "file, '$path_to_list': \n" . slurp($path_to_list);
@@ -1423,7 +1330,12 @@ SKIP: {
       $lh->create_mass_sending_file(
         -ID   => DADA::App::Guts::message_id(),    #argh. That's messy.
         -Type => 'list',
-        -partial_sending => { 'last_name' => { 'equal_to' => 'Duchamp' } },
+        -partial_sending => { 
+            last_name => { 
+                -operator => '=', 
+                -value    => 'Duchamp', 
+            } 
+        },
       );
 
     ok(
@@ -1440,7 +1352,7 @@ SKIP: {
       $lh->create_mass_sending_file(
         -ID   => DADA::App::Guts::message_id(),    #argh. That's messy.
         -Type => 'list',
-        -partial_sending => { 'first_name' => { 'like' => 'Ma' } },
+        -partial_sending => { 'first_name' => { -operator => 'LIKE', -value => 'Ma' } },
       );
 
     ok(
@@ -1470,7 +1382,7 @@ SKIP: {
       $lh->create_mass_sending_file(
         -ID   => DADA::App::Guts::message_id(),    #argh. That's messy.
         -Type => 'list',
-        -partial_sending => { 'first_name' => { 'equal_to' => 'Raymond' } },
+        -partial_sending => { 'first_name' => { -operator => '=', -value => 'Raymond' } },
       );
 
     ok(
@@ -1482,7 +1394,7 @@ SKIP: {
     undef($total_sending_out_num);
 
     my $r_count =
-      $lh->remove_from_list( -Email_List => ['raymond.lame@example.com'] );
+      $lh->remove_subscriber({ -email => 'raymond.lame@example.com' });
     ok( $r_count == 1, "removed one address (raymond.lame\@example.com)" );
     undef($r_count);
 
@@ -1537,38 +1449,38 @@ SKIP: {
     chmod( 0777, $DADA::Config::TMP . '/mail.txt' );
 
     ok(
-        $lh->remove_from_list(
-            -Email_List => ['mike.kelley@example.com'],
-            -Type       => 'list'
-        ),
+        $lh->remove_subscriber({
+            -email => 'mike.kelley@example.com',
+            -type       => 'list'
+        }),
         "removed: " . 'mike.kelley@example.com'
     );
     ok(
-        $lh->remove_from_list(
-            -Email_List => ['raymond.pettibon@example.com'],
-            -Type       => 'list'
-        ),
+        $lh->remove_subscriber({
+            -email => 'raymond.pettibon@example.com',
+            -type       => 'list'
+        }),
         "removed: " . 'raymond.pettibon@example.com'
     );
     ok(
-        $lh->remove_from_list(
-            -Email_List => ['marcel.duchamp@example.com'],
-            -Type       => 'list'
-        ),
+        $lh->remove_subscriber({
+            -email => 'marcel.duchamp@example.com',
+            -type       => 'list'
+        }),
         "removed: " . 'marcel.duchamp@example.com'
     );
     ok(
-        $lh->remove_from_list(
-            -Email_List => ['man.ray@example.com'],
-            -Type       => 'list'
-        ),
+        $lh->remove_subscriber({
+            -email => 'man.ray@example.com',
+            -type       => 'list'
+        }),
         "removed: " . 'man.ray@example.com'
     );
     ok(
-        $lh->remove_from_list(
-            -Email_List => ['no.one@example.com'],
-            -Type       => 'list'
-        ),
+        $lh->remove_subscriber({
+            -email => 'no.one@example.com',
+            -type       => 'list'
+        }),
         "removed: " . 'no.one@example.com'
     );
 
@@ -1614,12 +1526,13 @@ my $results     = 0;
 
 ( $num_results, $results ) = $lh->search_list(
     {
-        -query => 'example',
-        -type  => 'list',
+        -operator => '=',
+        -value    => 'example',
+        -type     => 'list',
     }
 );
 
-ok( $results->[9],   "Have 10 results (scalar @$results)" );
+ok( $results->[9],   "Have 10 results (" . scalar(@$results) . ")" );
 ok( !$results->[10], "Do NOT have 11 results." );
 ok($num_results == 10, "num results = 10 ($num_results)"); 
 
@@ -1630,10 +1543,10 @@ for (@$results) {
 $i = 0;
 for ( $i = 0 ; $i < 10 ; $i++ ) {
     ok(
-        $lh->remove_from_list(
-            -Email_List => [ 'example' . $i . '@example.com' ],
-            -Type       => 'list'
-          ) == 1,
+        $lh->remove_subscriber({
+            -email =>  'example' . $i . '@example.com' ,
+            -type       => 'list'
+          }) == 1,
         "removed: " . 'example' . $i . '@example.com'
     );
 }
@@ -1642,7 +1555,7 @@ undef $i;
 SKIP: {
 
     skip
-"Multiple Subscriber Profile Fields is not supported with this current backend."
+"Multiple Profile Fields is not supported with this current backend."
       if $lh->can_have_subscriber_fields == 0;
 
     for (qw(1 2)) {
@@ -1679,8 +1592,9 @@ SKIP: {
 
     ( $num_results, $results ) = $lh->search_list(
         {
-            -query => 'example',
-            -type  => 'list',
+            -operator => '=', 
+            -value   => 'example',
+            -type    => 'list',
         }
     );
 
@@ -1696,8 +1610,8 @@ SKIP: {
 
     ( $num_results, $results ) = $lh->search_list(
         {
-            -query => 'Jones',
-            -type  => 'list',
+            -query    => 'Jones',
+            -type     => 'list',
         }
     );
 
@@ -1710,8 +1624,8 @@ SKIP: {
 
     ( $num_results, $results ) = $lh->search_list(
         {
-            -query => 'Wes',
-            -type  => 'list',
+            -query    => 'Wes',
+            -type     => 'list',
         }
     );
 
@@ -1720,8 +1634,8 @@ SKIP: {
 
     ( $num_results, $results ) = $lh->search_list(
         {
-            -query => 'Wes',
-            -type  => 'list',
+            -query    => 'Wes',
+            -type     => 'list',
         }
     );
     ok( $results->[0],  "Found 1 result" );
@@ -1736,10 +1650,10 @@ SKIP: {
 
     for (@$subscribers) {
         ok(
-            $lh->remove_from_list(
-                -Email_List => [ $_->[0] ],
-                -Type       => 'list'
-            ),
+            $lh->remove_subscriber({
+                -email =>  $_->[0] ,
+                -type       => 'list'
+            }),
             "removed: " . $_->[0]
         );
     }
@@ -1806,7 +1720,7 @@ eval {
 ok( defined($@), "dupe check worked! died!" );
 like( $@, qr/unknown option/, "and the error message seems comprehensible!" );
 
-$lh->remove_from_list( -Email_List => [$dupe_email] );
+$lh->remove_subscriber({ -email => $dupe_email });
 
 ### /search_list
 
@@ -2119,12 +2033,12 @@ ok( $status == 1, "Status is 1 ($status)" );
 
 
 # List-specific quota: 
-$ls->save(
+$ls->save({ -settings => 
 	{ 
 		use_subscription_quota => 1, 
 		subscription_quota     => 1000, 
 	}	
-); 
+}); 
 my ( $status, $details ) = $lh->subscription_check(
     {
         -email => 'yetonemoresubscriber@example.com',
@@ -2136,12 +2050,12 @@ ok( $details->{over_subscription_quota} == 1, "over_subscription_quota");
 
 # List-specific quota, bigger than Global Quota - global quota should be used:  
 $DADA::Config::SUBSCRIPTION_QUOTA = 1000; 
-$ls->save(
+$ls->save({ -settings => 
 	{ 
 		use_subscription_quota => 1, 
 		subscription_quota     => 50000, 
 	}	
-); 
+}); 
 my ( $status, $details ) = $lh->subscription_check(
     {
         -email => 'yetonemoresubscriber@example.com',
@@ -2152,19 +2066,61 @@ ok( $status == 0, "Status is 0 ($status)" );
 ok( $details->{over_subscription_quota} == 1, "over_subscription_quota"); 
 
 #undef($subscribed, $not_subscribed, $black_listed, $not_white_listed, $invalid); 
-my ($subscribed, $not_subscribed, $black_listed, $not_white_listed, $invalid)
-	= $lh->filter_subscribers(
+my $f_emails
+	= $lh->filter_subscribers_w_meta(
 		{
-			-emails => ['yetonemoresubscriber@example.com'],
+			-emails => [{-email => 'yetonemoresubscriber@example.com'}],
 			-type   => 'list',
 		}
 	);
-ok($invalid->[0] eq 'yetonemoresubscriber@example.com'); 	
+#ok($invalid->[0] eq 'yetonemoresubscriber@example.com'); 	
 
 ok($lh->remove_all_subscribers( { -type => 'list' } ) == 1000);
 
 
 
+SKIP: {
+
+    skip
+"Multiple Profile Fields is not supported with this current backend."
+      if $lh->can_have_subscriber_fields == 0;
+      
+      
+      my $shh = $lh->add_subscriber_field( 
+          { 
+              -field    => '_new_field', 
+              -label    => "sssssssh!!!!",
+              -required => 1,
+          } 
+        );
+        ok( $shh == 1, "adding a new field is successful" );
+        
+        undef $lh; 
+        my $lh = DADA::MailingList::Subscribers->new( { -list => $list } );
+        
+        my $good_address = 'ssh.user@example.com'; 
+        # no -mode
+        ( $status, $details ) =  $lh->subscription_check( { -email => $good_address, } );
+        ok( $status == 1,                  "Status is 1" );
+        ok( $details->{invalid_email} != 1, "Address seen as valid" );
+        
+        # explicit -mode, 'user'
+        ( $status, $details ) =  $lh->subscription_check( { -email => $good_address, -mode => 'user'} );
+        ok( $status == 1,                  "Status is 1" );
+        ok( $details->{invalid_email} != 1, "Address seen as valid" );
+
+        # explicit -mode, 'admin'
+        ( $status, $details ) =  $lh->subscription_check( { -email => $good_address, -mode => 'admin'} );
+        
+        #use Data::Dumper; 
+        #diag Dumper([$status, $details]); 
+        ok( $status == 0,                  "Status is 0" );
+        ok( $details->{invalid_profile_fields}->{_new_field}->{required} == 1, "Didn't pass validation!" );
+        undef($good_address); 
+      
+      
+}; 
+
 dada_test_config::remove_test_list;
 dada_test_config::wipe_out;
 
