@@ -1,11 +1,11 @@
 package dada_test_config;
 
-use __Test_Config_Vars; 
 use Carp qw(croak carp); 
 #$Carp::Verbose = 1; 
-
+use lib '/Users/justin/Documents/DadaMail/git/dada-mail/dada/t'; 
 
 BEGIN { 
+    
     use FindBin '$Bin';
     use lib "$Bin/../ $Bin/";
 	use DADA::Config; 
@@ -28,10 +28,20 @@ $DADA::Config::PROGRAM_USAGE_LOG        = $DADA::Config::FILES . '/dada.txt';
 				croak "I couldn't make a tmp directory - heavens!"; 
 		}		
 	}
-	
+	$DADA::Config::SUPPORT_FILES = {
+	    dir => 'static',
+	    url => undef,
+	};
+
+
+	`mkdir static/themes`;
+	`mkdir static/themes/email`;
+	`mkdir static/themes/email/default`;
+	`cp -rX /Users/justin/Documents/DadaMail/git/dada_mail_foundation_email_templates/dist static/themes/email/default/dist`;
 
 }
 
+use __Test_Config_Vars; 
 
 use lib "$Bin/../DADA/perllib";
 use Params::Validate ':all';
@@ -79,7 +89,7 @@ use strict;
 sub test_list_vars { 
 
 
-    my $foo = { 
+    my $test_list_vars = { 
     
             list             => 'dadatest', 
             list_name        => 'Dada Test List' . $UTF8_STR, 
@@ -92,7 +102,7 @@ sub test_list_vars {
     
     };
 
-    return $foo; 
+    return $test_list_vars; 
 
 }
 
@@ -130,7 +140,7 @@ sub create_test_list {
 	); 
    
     if($args{-remove_subscriber_fields} == 1){ 
-        #carp 'Removing extraneous Subscriber Profile Fields....'; 
+        #carp 'Removing extraneous Profile Fields....'; 
         require DADA::MailingList::Subscribers; 
         my $lh = DADA::MailingList::Subscribers->new({-list => $list_name}); 
         my $fields = $lh->subscriber_fields;
@@ -212,7 +222,8 @@ my @statements = split(';', $sql);
     my $dbh = $dbi_handle->dbh_obj;
     
     for(@statements){ 
-			
+		print $_ . "\n"; 
+		
     	my $settings_table                      = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{settings_table}; 
 		my $subscribers_table    	            = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{subscriber_table}; 
 		my $archives_table          		    = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{archives_table}; 
@@ -221,13 +232,21 @@ my @statements = split(';', $sql);
 		my $profile_table            		    = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{profile_table};  
 		my $profile_fields_table                = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{profile_fields_table};
 		my $profile_fields_attributes_table     = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{profile_fields_attributes_table};
+		my $profile_settings_table              = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{profile_settings_table};
 		my $clickthrough_urls_table             = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{clickthrough_urls_table};
 		my $clickthrough_url_log_table          = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{clickthrough_url_log_table};
 		my $mass_mailing_event_log_table        = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{mass_mailing_event_log_table};
 		my $password_protect_directories_table  = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{password_protect_directories_table};
 		my $confirmation_tokens_table           = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{confirmation_tokens_table};
 		my $message_drafts_table                = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{message_drafts_table};
-
+		my $rate_limit_hits_table               = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{rate_limit_hits_table}; 
+		my $email_message_previews_table        = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{email_message_previews_table}; 
+		
+		my $privacy_policies_table              = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{privacy_policies_table};
+		my $consents_table                      = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{consents_table};
+		my $consent_activity_table              = $__Test_Config_Vars::TEST_SQL_PARAMS->{SQLite}->{consent_activity_table};
+		
+						
 						
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_settings}{CREATE TABLE IF NOT EXISTS $settings_table}; 
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_subscribers}{CREATE TABLE IF NOT EXISTS $subscribers_table}; 
@@ -237,18 +256,25 @@ my @statements = split(';', $sql);
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_profiles}{CREATE TABLE IF NOT EXISTS $profile_table};
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_profile_fields}{CREATE TABLE IF NOT EXISTS $profile_fields_table};
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_profile_fields_attributes}{CREATE TABLE IF NOT EXISTS $profile_fields_attributes_table};
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_profile_settings}{CREATE TABLE $profile_settings_table};	
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_clickthrough_urls}{CREATE TABLE IF NOT EXISTS $clickthrough_urls_table};
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_clickthrough_url_log}{CREATE TABLE IF NOT EXISTS $clickthrough_url_log_table};	
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_mass_mailing_event_log}{CREATE TABLE IF NOT EXISTS $mass_mailing_event_log_table};	
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_password_protect_directories}{CREATE TABLE IF NOT EXISTS $password_protect_directories_table};	
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_confirmation_tokens}{CREATE TABLE IF NOT EXISTS $confirmation_tokens_table};	
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_message_drafts}{CREATE TABLE IF NOT EXISTS $message_drafts_table};	
-		
-		print 'query: ' . $_ . "\n\n"; 
-        my $sth = $dbh->prepare($_) or croak $DBI::errstr; 
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_rate_limit_hits}{CREATE TABLE IF NOT EXISTS $rate_limit_hits_table};	
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_email_message_previews}{CREATE TABLE IF NOT EXISTS $email_message_previews_table};	
+
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_privacy_policies}{CREATE TABLE IF NOT EXISTS $privacy_policies_table};	
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_consents}{CREATE TABLE IF NOT EXISTS $consents_table};	
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_consent_activity}{CREATE TABLE IF NOT EXISTS $consent_activity_table};	
+
+
+        my $sth = $dbh->prepare($_) or croak $DBI::errstr . 'query:' . $_; 
 
        $sth->execute
-			or croak "cannot do statment $DBI::errstr\n"; 
+			or croak "cannot do statement $DBI::errstr\n"; 
 			#sleep(1);
     }
 	# print "Sleepin!"; 
@@ -317,12 +343,18 @@ my @statements = split(';', $sql);
 		my $profile_table            		    = $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{profile_table};  
 		my $profile_fields_table                = $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{profile_fields_table};
 		my $profile_fields_attributes_table     = $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{profile_fields_attributes_table};
+		my $profile_settings_table              = $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{profile_settings_table};
 		my $clickthrough_urls_table             = $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{clickthrough_urls_table};
 		my $clickthrough_url_log_table          = $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{clickthrough_url_log_table};
 		my $mass_mailing_event_log_table        = $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{mass_mailing_event_log_table};
 		my $password_protect_directories_table  = $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{password_protect_directories_table};
 		my $confirmation_tokens_table           = $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{confirmation_tokens_table};
 		my $message_drafts_table                = $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{message_drafts_table};
+		my $rate_limit_hits_table               = $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{rate_limit_hits_table}; 
+		my $email_message_previews_table        = $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{email_message_previews_table}; 
+		my $privacy_policies_table              = $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{privacy_policies_table};
+		my $consents_table                      = $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{consents_table};
+		my $consent_activity_table              = $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{consent_activity_table};
 		
 		
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_settings}{CREATE TABLE $settings_table}; 
@@ -333,6 +365,7 @@ my @statements = split(';', $sql);
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_profiles}{CREATE TABLE $profile_table};
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_profile_fields}{CREATE TABLE $profile_fields_table};
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_profile_fields_attributes}{CREATE TABLE $profile_fields_attributes_table};	
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_profile_settings}{CREATE TABLE $profile_settings_table};	
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_clickthrough_urls}{CREATE TABLE $clickthrough_urls_table};	
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_settings}{CREATE TABLE IF NOT EXISTS $settings_table}; 
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_subscribers}{CREATE TABLE IF NOT EXISTS $subscribers_table}; 
@@ -347,6 +380,12 @@ my @statements = split(';', $sql);
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_password_protect_directories}{CREATE TABLE IF NOT EXISTS $password_protect_directories_table};	
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_confirmation_tokens}{CREATE TABLE IF NOT EXISTS $confirmation_tokens_table};	
 		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_message_drafts}{CREATE TABLE IF NOT EXISTS $message_drafts_table};	
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_rate_limit_hits}{CREATE TABLE IF NOT EXISTS $rate_limit_hits_table};	
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_email_message_previews}{CREATE TABLE IF NOT EXISTS $email_message_previews_table};	
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_privacy_policies}{CREATE TABLE IF NOT EXISTS $privacy_policies_table};	
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_consents}{CREATE TABLE IF NOT EXISTS $consents_table};	
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_consent_activity}{CREATE TABLE IF NOT EXISTS $consent_activity_table};	
+
 
 		#print 'query: ' . $_; 
 			
@@ -355,7 +394,7 @@ my @statements = split(';', $sql);
 			#warn 'QUERY: ' . $_; 
 			
 			my $sth = $dbh->prepare($_); 
-	       	$sth->execute; 
+	       	$sth->execute;#or die $DBI::errstr; 
 	    }
     
     }
@@ -364,7 +403,7 @@ my @statements = split(';', $sql);
 }
 
 sub destroy_MySQL_db { 
-
+	
  #	carp 'destroy_MySQL_db1'; 
 #	warn 'destroy_MySQL_db2';
 	 
@@ -381,6 +420,7 @@ sub destroy_MySQL_db {
 		bounce_scores_table
 		profile_table
 		profile_fields_table
+		profile_settings_table
 		profile_fields_attributes_table
 		clickthrough_urls_table
 		clickthrough_url_log_table
@@ -388,6 +428,11 @@ sub destroy_MySQL_db {
 		password_protect_directories_table
 		confirmation_tokens_table
 		message_drafts_table
+		rate_limit_hits_table
+		email_message_previews_table
+		privacy_policies_table 
+		consents_table         
+		consent_activity_table 
 		)){ 
 			
 #			carp "removing: " . $__Test_Config_Vars::TEST_SQL_PARAMS->{MySQL}->{$_}; 
@@ -442,20 +487,26 @@ my @statements = split(';', $sql);
     for(@statements){ 
    
 
-	   	my $settings_table                   = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{settings_table}; 
-		my $subscribers_table    	         = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{subscriber_table}; 
-		my $archives_table          		 = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{archives_table}; 
-		my $session_table           		 = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{session_table};
-		my $bounce_scores_table     		 = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{bounce_scores_table};
-		my $profile_table            		 = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{profile_table};  
+	   	my $settings_table                      = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{settings_table}; 
+		my $subscribers_table    	            = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{subscriber_table}; 
+		my $archives_table          		    = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{archives_table}; 
+		my $session_table                       = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{session_table};
+		my $bounce_scores_table     		    = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{bounce_scores_table};
+		my $profile_table            		    = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{profile_table};  
 		my $profile_fields_table                = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{profile_fields_table};
 		my $profile_fields_attributes_table     = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{profile_fields_attributes_table};
+	    my $profile_settings_table              = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{profile_settings_table};
 		my $clickthrough_urls_table             = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{clickthrough_urls_table};
 		my $clickthrough_url_log_table          = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{clickthrough_url_log_table};
 		my $mass_mailing_event_log_table        = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{mass_mailing_event_log_table};
 		my $password_protect_directories_table  = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{password_protect_directories_table};
 		my $confirmation_tokens_table           = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{confirmation_tokens_table};
 		my $message_drafts_table                = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{message_drafts_table};
+		my $rate_limit_hits_table               = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{rate_limit_hits_table}; 
+		my $email_message_previews_table        = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{email_message_previews_table}; 
+		my $privacy_policies_table              = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{privacy_policies_table};
+		my $consents_table                      = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{consents_table};
+		my $consent_activity_table              = $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{consent_activity_table};
 	
 		
 		
@@ -468,14 +519,21 @@ my @statements = split(';', $sql);
 		$_ =~ s{CREATE TABLE dada_profiles}{CREATE TABLE $profile_table};
 		$_ =~ s{CREATE TABLE dada_profile_fields}{CREATE TABLE $profile_fields_table};
 		$_ =~ s{CREATE TABLE dada_profile_fields_attributes}{CREATE TABLE $profile_fields_attributes_table};
+		$_ =~ s{CREATE TABLE dada_profile_settings}{CREATE TABLE $profile_settings_table};	
 		$_ =~ s{CREATE TABLE dada_clickthrough_urls}{CREATE TABLE $clickthrough_urls_table};	
 		$_ =~ s{CREATE TABLE dada_clickthrough_url_log}{CREATE TABLE $clickthrough_url_log_table};	
 		$_ =~ s{CREATE TABLE dada_mass_mailing_event_log}{CREATE TABLE $mass_mailing_event_log_table};	
 		$_ =~ s{CREATE TABLE dada_password_protect_directories}{CREATE TABLE $password_protect_directories_table};	
 		$_ =~ s{CREATE TABLE dada_confirmation_tokens}{CREATE TABLE IF NOT EXISTS $confirmation_tokens_table};	
 		$_ =~ s{CREATE TABLE dada_message_drafts}{CREATE TABLE IF NOT EXISTS $message_drafts_table};	
+		$_ =~ s{CREATE TABLE dada_rate_limit_hits}{CREATE TABLE IF NOT EXISTS $rate_limit_hits_table};	
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_email_message_previews}{CREATE TABLE IF NOT EXISTS $email_message_previews_table};	
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_privacy_policies}{CREATE TABLE IF NOT EXISTS $privacy_policies_table};	
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_consents}{CREATE TABLE IF NOT EXISTS $consents_table};	
+		$_ =~ s{CREATE TABLE IF NOT EXISTS dada_consent_activity}{CREATE TABLE IF NOT EXISTS $consent_activity_table};	
 
-#		print "query: $_"; 
+
+		print "query: $_"; 
 
 
 	    my $sth = $dbh->prepare($_); #  or croak $DBI::errstr; 
@@ -502,12 +560,19 @@ sub destroy_PostgreSQL_db {
 		profile_table
 		profile_fields_table
 		profile_fields_attributes_table
+		profile_settings_table
 		clickthrough_urls_table
 		clickthrough_url_log_table
 		mass_mailing_event_log_table
 		password_protect_directories_table
 		confirmation_tokens_table
 		message_drafts_table
+		rate_limit_hits_table
+		email_message_previews_table
+		privacy_policies_table 
+		consents_table         
+		consent_activity_table 
+		
 		)){ 
 	        $dbh->do('DROP TABLE ' . $__Test_Config_Vars::TEST_SQL_PARAMS->{PostgreSQL}->{$_})
 	            or carp "cannot do statement! $DBI::errstr\n";  
@@ -525,6 +590,9 @@ sub wipe_out {
 	if(-e './test_only_dada_files'){ 
 		warn "wiping out didn't work!"; 
 	}
+	
+	`rm -rf static/themes`;
+	
 }
 
 sub MySQL_test_enabled { 
@@ -557,4 +625,4 @@ sub slurp {
 }
 
 
-
+1;
